# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


try:
    from PyQt5 import Qt, QtCore, QtGui, QtWidgets
except ImportError:
    raise SystemExit(
        'Please install PyQt5 to run this script (http://www.riverbankcomputing.co.uk/software/pyqt/download)')


# Movable solid line for filter ideal-band diagram.
# Enable split to cut the line into two (for bpf).
class filtermovlineItem(QtWidgets.QGraphicsObject):
    attenChanged = QtCore.pyqtSignal(float)

    def __init__(self, x1, y1, x2, y2, lower, upper, split=False, sx1=0, sy1=0, sx2=0, sy2=0):
        QtWidgets.QGraphicsObject.__init__(self)
        self.lower = lower
        self.upper = upper
        self.x1, self.y1 = x1, y1
        self.x2, self.y2 = x2, y2
        self.sx1, self.sy1 = sx1, sy1
        self.sx2, self.sy2 = sx2, sy2
        self.split = split

    def paint(self, painter, option, widget):
        painter.setPen(QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine))
        painter.drawLine(self.x1, self.y1, self.x2, self.y2)
        painter.drawLine(self.x1, self.y1, self.x1, self.y1 - 5)
        painter.drawLine(self.x2, self.y2, self.x2, self.y2 - 5)
        if self.split:
            painter.drawLine(self.sx1, self.sy1, self.sx2, self.sy2)
            painter.drawLine(self.sx1, self.sy1, self.sx1, self.sy1 - 5)
            painter.drawLine(self.sx2, self.sy2, self.sx2, self.sy2 - 5)

    def boundingRect(self):
        return QtCore.QRectF(0, 0, 400, 400)

    # Allow only vertical movement and emit signals.
    def itemChange(self, change, value):
        if (change == QtWidgets.QGraphicsItem.ItemPositionChange):
            newpos = QtCore.QPointF(value)
            div = 0
            if newpos.y() < self.pos().y():
                div = 1
            if newpos.y() > self.pos().y():
                div = -1
            hit = 0
            rect = self.scene().sceneRect()
            if (newpos.y() >= self.lower):
                newpos.setY(self.lower)
                hit = 1
            if (newpos.y() <= self.upper):
                newpos.setY(self.upper)
                hit = 1
            if not(hit):
                self.attenChanged.emit(div)
            return QtCore.QPointF(self.pos().x(), newpos.y())
        return QtWidgets.QGraphicsItem.itemChange(self, change, value)


# Static lines in lpf band diagram.
class lpfsLines(QtWidgets.QGraphicsObject):
    def __init__(self):
        QtWidgets.QGraphicsObject.__init__(self)

    def paint(self, painter, option, widget):
        painter.setPen(QtGui.QPen(QtCore.Qt.darkGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(5, 20, 5, 200)
        painter.drawLine(5, 200, 400, 200)
        painter.setPen(QtGui.QPen(QtCore.Qt.lightGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(6, 105, 150, 105)
        painter.drawLine(150, 105, 150, 199)
        painter.drawLine(200, 180, 400, 180)
        painter.drawLine(200, 180, 200, 199)
        painter.setPen(QtGui.QPen(QtCore.Qt.black))
        painter.save()
        painter.rotate(270)
        painter.drawText(-150, -10, "Magnitude (dB)")
        painter.restore()
        painter.drawText(350, 220, "Frequency (Hz)")
        painter.drawText(130, 220, "Fpass")
        painter.drawText(180, 220, "Fstop")

    def boundingRect(self):
        return QtCore.QRectF(0, 0, 300, 300)


# Static lines in hpf band diagram.
class hpfsLines(QtWidgets.QGraphicsObject):
    def __init__(self):
        QtWidgets.QGraphicsObject.__init__(self)

    def paint(self, painter, option, widget):
        painter.setPen(QtGui.QPen(QtCore.Qt.darkGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(5, 20, 5, 200)
        painter.drawLine(5, 200, 400, 200)
        painter.setPen(QtGui.QPen(QtCore.Qt.lightGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(200, 105, 400, 105)
        painter.drawLine(200, 105, 200, 199)
        painter.drawLine(6, 180, 150, 180)
        painter.drawLine(150, 180, 150, 199)
        painter.setPen(QtGui.QPen(QtCore.Qt.black))
        painter.drawText(350, 220, "Frequency (Hz)")
        painter.drawText(130, 220, "Fstop")
        painter.drawText(180, 220, "Fpass")

    def boundingRect(self):
        return QtCore.QRectF(0, 0, 300, 300)


# Static lines in bpf band diagram.
class bpfsLines(QtWidgets.QGraphicsObject):
    def __init__(self):
        QtWidgets.QGraphicsObject.__init__(self)

    def paint(self, painter, option, widget):
        painter.setPen(QtGui.QPen(QtCore.Qt.darkGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(5, 20, 5, 200)
        painter.drawLine(5, 200, 400, 200)
        painter.setPen(QtGui.QPen(QtCore.Qt.lightGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(6, 180, 110, 180)
        painter.drawLine(110, 180, 110, 199)
        painter.drawLine(155, 105, 255, 105)
        painter.drawLine(255, 105, 255, 199)
        painter.drawLine(155, 105, 155, 199)
        painter.drawLine(300, 180, 400, 180)
        painter.drawLine(300, 180, 300, 199)
        painter.setPen(QtGui.QPen(QtCore.Qt.black))
        painter.drawText(350, 220, "Frequency (Hz)")
        painter.drawText(80, 220, "Fstop1")
        painter.drawText(140, 220, "Fpass1")
        painter.drawText(220, 220, "Fpass2")
        painter.drawText(290, 220, "Fstop2")

    def boundingRect(self):
        return QtCore.QRectF(0, 0, 300, 300)


# Static lines in bnf band diagram.
class bnfsLines(QtWidgets.QGraphicsObject):
    def __init__(self):
        QtWidgets.QGraphicsObject.__init__(self)

    def paint(self, painter, option, widget):
        painter.setPen(QtGui.QPen(QtCore.Qt.darkGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(5, 20, 5, 200)
        painter.drawLine(5, 200, 400, 200)
        painter.setPen(QtGui.QPen(QtCore.Qt.lightGray, 2, QtCore.Qt.SolidLine))
        painter.drawLine(6, 105, 110, 105)
        painter.drawLine(110, 105, 110, 199)
        painter.drawLine(155, 180, 255, 180)
        painter.drawLine(255, 180, 255, 199)
        painter.drawLine(155, 180, 155, 199)
        painter.drawLine(300, 105, 400, 105)
        painter.drawLine(300, 105, 300, 199)
        painter.setPen(QtGui.QPen(QtCore.Qt.black))
        painter.drawText(350, 220, "Frequency (Hz)")
        painter.drawText(80, 220, "Fpass1")
        painter.drawText(140, 220, "Fstop1")
        painter.drawText(220, 220, "Fstop2")
        painter.drawText(290, 220, "Fpass2")

    def boundingRect(self):
        return QtCore.QRectF(0, 0, 300, 300)


lpfItems = []
hpfItems = []
bpfItems = []
bnfItems = []

# lpfitems list.
lpfItems.append(filtermovlineItem(200, 175, 400, 175, 0, -60))
# lpfItems.append(filtermovlineItem(200, 145, 400, 145, 30, -30))
lpfItems[0].setFlags(QtWidgets.QGraphicsItem.ItemIsSelectable | QtWidgets.QGraphicsItem.ItemIsMovable |
                     QtWidgets.QGraphicsItem.ItemSendsGeometryChanges)
lpfItems.append(lpfsLines())

# hpfitems list.
hpfItems.append(filtermovlineItem(6, 175, 150, 175, 0, -60))
hpfItems[0].setFlags(QtWidgets.QGraphicsItem.ItemIsSelectable | QtWidgets.QGraphicsItem.ItemIsMovable |
                     QtWidgets.QGraphicsItem.ItemSendsGeometryChanges)
hpfItems.append(hpfsLines())

# bpfitems list.
bpfItems.append(filtermovlineItem(6, 175, 110, 175,
                0, -60, True, 300, 175, 400, 175))
bpfItems[0].setFlags(QtWidgets.QGraphicsItem.ItemIsSelectable | QtWidgets.QGraphicsItem.ItemIsMovable |
                     QtWidgets.QGraphicsItem.ItemSendsGeometryChanges)
bpfItems.append(bpfsLines())

# bnfitems list.
bnfItems.append(filtermovlineItem(155, 175, 255, 175, 0, -60))
bnfItems[0].setFlags(QtWidgets.QGraphicsItem.ItemIsSelectable | QtWidgets.QGraphicsItem.ItemIsMovable |
                     QtWidgets.QGraphicsItem.ItemSendsGeometryChanges)
bnfItems.append(bnfsLines())



#!/usr/bin/env python
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, vocoder, blocks


class test_alaw_vocoder (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test001_module_load(self):
        data = (8, 24, 40, 56, 72, 88, 104, 120, 136, 152, 168, 184,
                200, 216, 232, 248, 264, 280, 296, 312, 328, 344)
        src = blocks.vector_source_s(data)
        enc = vocoder.alaw_encode_sb()
        dec = vocoder.alaw_decode_bs()
        snk = blocks.vector_sink_s()
        self.tb.connect(src, enc, dec, snk)
        self.tb.run()
        actual_result = snk.data()
        self.assertEqual(list(data), actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_alaw_vocoder)



# Copyright 2017 Free Software Foundation, Inc.
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-2.0-or-later
#

from os import path
import sys

from grc.converter import flow_graph


def test_flow_graph_converter():
    filename = path.join(path.dirname(__file__),
                         'resources', 'test_compiler.grc')

    data = flow_graph.from_xml(filename)

    flow_graph.dump(data, sys.stdout)


def test_flow_graph_converter_with_fp():
    filename = path.join(path.dirname(__file__),
                         'resources', 'test_compiler.grc')

    with open(filename, 'rb') as fp:
        data = flow_graph.from_xml(fp)

    flow_graph.dump(data, sys.stdout)



#!/usr/bin/env /usr/bin/python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from gnuradio import gr, filter, analog
from .atsc_rx_filter import *


class atsc_rx(gr.hier_block2):
    def __init__(self, input_rate, sps):
        gr.hier_block2.__init__(self, "atsc_rx",
                                # Input signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(1, 1, gr.sizeof_char))       # Output signature

        # ATSC receiver filter/interpolator
        rx_filt = atsc_rx_filter(input_rate, sps)

        # Lock on to pilot tone, shift to DC, then discard Q channel
        output_rate = ATSC_SYMBOL_RATE * sps
        pll = dtv.atsc_fpll(output_rate)

        # Remove pilot tone now at DC
        dcr = filter.dc_blocker_ff(4096)

        # Normalize signal to proper constellation amplitude
        agc = analog.agc_ff(1e-5, 4.0)

        # Synchronize bit and segment timing
        btl = dtv.atsc_sync(output_rate)

        # Check for correct field sync
        fsc = dtv.atsc_fs_checker()

        # Equalize channel using training sequences
        equ = dtv.atsc_equalizer()

        # Remove convolutional trellis coding
        vit = dtv.atsc_viterbi_decoder()

        # Remove convolutional interleaving
        dei = dtv.atsc_deinterleaver()

        # Reed-Solomon decode
        rsd = dtv.atsc_rs_decoder()

        # Derandomize MPEG2-TS packet
        der = dtv.atsc_derandomizer()

        # Remove padding from packet
        dep = dtv.atsc_depad()

        # Connect pipeline
        self.connect(self, rx_filt, pll, dcr, agc, btl, fsc)
        self.connect((fsc, 0), (equ, 0))
        self.connect((fsc, 1), (equ, 1))
        self.connect((equ, 0), (vit, 0))
        self.connect((equ, 1), (vit, 1))
        self.connect((vit, 0), (dei, 0))
        self.connect((vit, 1), (dei, 1))
        self.connect((dei, 0), (rsd, 0))
        self.connect((dei, 1), (rsd, 1))
        self.connect((rsd, 0), (der, 0))
        self.connect((rsd, 1), (der, 1))
        self.connect((der, 0), (dep, 0))
        self.connect((dep, 0), (self, 0))



#!/usr/bin/env python
#
# Copyright 2008,2010,2012,2013 Free Software Foundation, Inc.
# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import time

from gnuradio import gr, gr_unittest, blocks, pdu
import numpy as np
import pmt


class test_repeat(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def run_fg(self, N, r, msg=""):
        src_data = np.arange(N)
        src = blocks.vector_source_i(src_data)
        rpt = blocks.repeat(gr.sizeof_int, r)
        dst = blocks.vector_sink_i()
        self.tb.connect(src, rpt, dst)
        self.tb.run()
        self.assertTrue(np.array_equal(dst.data(), src_data.repeat(r)), msg)

    def test_001_various_sizes(self):
        sizes = ((3**10 + 2, 3), (3**6, 1), (4, 5), (10**6, 1), (10 * 2, 10 * 3))
        for size, repetitions in sizes:
            self.run_fg(size, repetitions, f"N = {size}, r = {repetitions}: not equal")

    def test_tag_propagation(self):
        N = 1000
        r = 17
        tags_in = [
            gr.tag_utils.python_to_tag((
                j, pmt.intern("test_tag"), pmt.from_long(j), pmt.PMT_NIL))
            for j in range(N)
        ]
        src = blocks.vector_source_i([0] * N, tags=tags_in)
        rpt = blocks.repeat(gr.sizeof_int, r)
        dst = blocks.vector_sink_i()
        self.tb.connect(src, rpt, dst)
        self.tb.run()
        tags_out = dst.tags()
        self.assertEqual(len(tags_in), len(tags_out))
        for t_in, t_out in zip(tags_in, tags_out):
            self.assertEqual(t_out.offset, r * t_in.offset)
            self.assertEqual(t_in.key, t_out.key)
            self.assertEqual(t_in.value, t_out.value)

    def test_tag_propagation_rate_change(self):
        # This uses a pdu_to_tagged_stream block to control when data is
        # produced, allowing us to change the repetition rate exactly when we
        # want.
        N1 = 1000
        r1 = 17
        N2 = 2000
        r2 = 13
        tags_in = [
            gr.tag_utils.python_to_tag((
                j, pmt.intern("test_tag"), pmt.from_long(j), pmt.PMT_NIL))
            for j in range(N1 + N2)
        ]
        src_tags = blocks.vector_source_b([0] * (N1 + N2), tags=tags_in)
        src_data = pdu.pdu_to_tagged_stream(gr.types.byte_t)
        tag_gate = blocks.tag_gate(gr.sizeof_char)
        tag_share = blocks.tag_share(gr.sizeof_char, gr.sizeof_char)
        rpt = blocks.repeat(gr.sizeof_char, r1)
        dst = blocks.vector_sink_b()
        self.tb.connect(src_data, tag_gate, tag_share, rpt, dst)
        self.tb.connect(src_tags, (tag_share, 1))
        self.tb.start()
        data_pdu = pmt.cons(pmt.PMT_NIL, pmt.make_u8vector(N1, 0))
        src_data.to_basic_block()._post(pmt.intern("pdus"), data_pdu)
        # Wait for some time for the data to flow through
        time.sleep(0.1)
        rpt.set_interpolation(r2)
        data_pdu = pmt.cons(pmt.PMT_NIL, pmt.make_u8vector(N2, 0))
        src_data.to_basic_block()._post(pmt.intern("pdus"), data_pdu)
        src_data.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))
        self.tb.wait()
        self.assertEqual(len(dst.data()), N1 * r1 + N2 * r2)
        tags_out = dst.tags()
        self.assertEqual(len(tags_out), len(tags_in))
        for t_in, t_out in zip(tags_in[:N1], tags_out[:N1]):
            self.assertEqual(t_out.offset, r1 * t_in.offset)
            self.assertEqual(t_in.key, t_out.key)
            self.assertEqual(t_in.value, t_out.value)
        for t_in, t_out in zip(tags_in[N1:], tags_out[N1:]):
            self.assertEqual(t_out.offset, N1 * r1 + r2 * (t_in.offset - N1))
            self.assertEqual(t_in.key, t_out.key)
            self.assertEqual(t_in.value, t_out.value)


if __name__ == "__main__":
    gr_unittest.run(test_repeat)



#!/usr/bin/env python3
#
# Copyright 2022 Johannes Demel
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gnuradio import gr, gr_unittest, blocks
import numpy as np


class test_blockinterleaver_xx(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()
        np.random.seed(0x13375eed)

    def tearDown(self):
        self.tb = None

    def test_001_init(self):
        nbits = 16 * 2
        interleaver_indices = np.random.permutation(nbits)

        # Test if packed interleaver catches byte multiple errors.
        self.assertRaises(
            Exception,
            blocks.blockinterleaver_bb,
            [np.sort(interleaver_indices)[0:13], True, True],
        )

        # Test if interleaver detects malformed interleaver indices.
        self.assertRaises(
            Exception,
            blocks.blockinterleaver_bb,
            [interleaver_indices[0:13], True, True],
        )

        # Make sure output multiple is set correctly!
        interleaver = blocks.blockinterleaver_bb(interleaver_indices, True, True)
        self.assertEqual(interleaver.output_multiple(), nbits // 8)
        self.assertTrue(interleaver.is_packed())
        self.assertTrue(interleaver.interleaver_mode())
        self.assertTupleEqual(
            tuple(interleaver.interleaver_indices()), tuple(interleaver_indices)
        )

        self.assertEqual(
            np.array(interleaver.deinterleaver_indices()).size, interleaver_indices.size
        )

        interleaveru = blocks.blockinterleaver_bb(interleaver_indices, False, False)
        self.assertEqual(interleaveru.output_multiple(), nbits)
        self.assertFalse(interleaveru.is_packed())
        self.assertFalse(interleaveru.interleaver_mode())

    def test_002_bb_unpacked(self):
        nframes = 5
        nbits = 16 * 7
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_bb(interleaver_indices, True, False)

        data = np.random.randint(0, 2, nbits * nframes)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_b(data)
        snk = blocks.vector_sink_b()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_003_bb_packed(self):
        nframes = 20
        nbits = 16 * 17
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_bb(interleaver_indices, True, True)

        data = np.random.randint(0, 2, nbits * nframes)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        datavec = np.packbits(data)
        refvec = np.packbits(ref)
        src = blocks.vector_source_b(datavec)
        snk = blocks.vector_sink_b()
        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # check data
        resvec = np.array(snk.data())

        self.assertTupleEqual(tuple(resvec), tuple(refvec))

    def test_004_ff_unpacked(self):
        nframes = 5
        nbits = 16 * 7
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_ff(interleaver_indices, True)

        data = np.random.normal(size=nbits * nframes).astype(np.float32)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_f(data)
        snk = blocks.vector_sink_f()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_005_ff_unpacked_deinterleave(self):
        nframes = 5
        nbits = 16 * 17
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_ff(interleaver_indices, False)

        data = np.random.normal(size=nbits * nframes).astype(np.float32)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_f(ref)
        snk = blocks.vector_sink_f()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(data))

    def test_006_cc_unpacked(self):
        nframes = 5
        nbits = 16 * 7
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_cc(interleaver_indices, True)

        data = np.random.normal(size=nbits * nframes).astype(
            np.float32
        ) + 1.0j * np.random.normal(size=nbits * nframes).astype(np.float32)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_c(data)
        snk = blocks.vector_sink_c()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_007_ss_unpacked(self):
        nframes = 5
        nbits = 16 * 7
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_ss(interleaver_indices, True)

        data = np.random.normal(size=nbits * nframes).astype(np.int16)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_s(data)
        snk = blocks.vector_sink_s()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_008_ii_unpacked(self):
        nframes = 5
        nbits = 16 * 7
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_ii(interleaver_indices, True)

        data = np.random.normal(size=nbits * nframes).astype(np.int32)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_i(data)
        snk = blocks.vector_sink_i()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_009_bb_unpacked_huge(self):
        nframes = 2
        nbits = int(2 ** 16) + int(2 ** 15)
        interleaver_indices = np.random.permutation(nbits)

        interleaver = blocks.blockinterleaver_bb(interleaver_indices, True, False)

        data = np.random.randint(0, 2, nbits * nframes)
        ref = np.array([], dtype=data.dtype)
        for f in np.reshape(data, (nframes, -1)):
            ref = np.concatenate((ref, f[interleaver_indices]))
        src = blocks.vector_source_b(data)
        snk = blocks.vector_sink_b()

        self.tb.connect(src, interleaver, snk)
        # set up fg
        self.tb.run()
        # # check data
        res = np.array(snk.data())
        self.assertTupleEqual(tuple(res), tuple(ref))


if __name__ == "__main__":
    gr_unittest.run(test_blockinterleaver_xx)



#!/usr/bin/env python
#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter
import sys
import math

# ----------------------------------------------------------------
# See optfir for an explanation of these.


def stopband_atten_to_dev(atten_db):
    """Convert a stopband attenuation in dB to an absolute value"""
    return 10**(-atten_db / 20)


def passband_ripple_to_dev(ripple_db):
    """Convert passband ripple spec expressed in dB to an absolute value"""
    return (10**(ripple_db / 20) - 1) / (10**(ripple_db / 20) + 1)

# ----------------------------------------------------------------


def remezord(fcuts, mags, devs, fsamp=2):
    '''
    FIR order estimator (lowpass, highpass, bandpass, mulitiband).
    '''

    # get local copies
    fcuts = fcuts[:]
    mags = mags[:]
    devs = devs[:]

    for i in range(len(fcuts)):
        fcuts[i] = float(fcuts[i]) / fsamp

    nf = len(fcuts)
    nm = len(mags)
    nd = len(devs)
    nbands = nm

    if nm != nd:
        raise ValueError("Length of mags and devs must be equal")

    if nf != 2 * (nbands - 1):
        raise ValueError("Length of f must be 2 * len (mags) - 2")

    for i in range(len(mags)):
        if mags[i] != 0:                        # if not stopband, get relative deviation
            devs[i] = devs[i] / mags[i]

    # separate the passband and stopband edges
    f1 = fcuts[0::2]
    f2 = fcuts[1::2]

    n = 0
    min_delta = 2
    for i in range(len(f1)):
        if f2[i] - f1[i] < min_delta:
            n = i
            min_delta = f2[i] - f1[i]

    if nbands == 2:
        # lowpass or highpass case (use formula)
        l = lporder(f1[n], f2[n], devs[0], devs[1])
    else:
        # bandpass or multipass case
        # try different lowpasses and take the worst one that
        #  goes through the BP specs
        l = 0
        for i in range(1, nbands - 1):
            l1 = lporder(f1[i - 1], f2[i - 1], devs[i], devs[i - 1])
            l2 = lporder(f1[i], f2[i], devs[i], devs[i + 1])
            l = max(l, l1, l2)

    n = int(math.ceil(l)) - 1               # need order, not length for remez

    # cook up remez compatible result
    ff = [0] + fcuts + [1]
    for i in range(1, len(ff) - 1):
        ff[i] *= 2

    aa = []
    for a in mags:
        aa = aa + [a, a]

    max_dev = max(devs)
    wts = [1] * len(devs)
    for i in range(len(wts)):
        wts[i] = max_dev / devs[i]

    return (n, ff, aa, wts)


def lporder(freq1, freq2, delta_p, delta_s):
    '''
    FIR lowpass filter length estimator.
    '''
    df = abs(freq2 - freq1)
    ddp = math.log10(delta_p)
    dds = math.log10(delta_s)

    a1 = 5.309e-3
    a2 = 7.114e-2
    a3 = -4.761e-1
    a4 = -2.66e-3
    a5 = -5.941e-1
    a6 = -4.278e-1

    b1 = 11.01217
    b2 = 0.5124401

    t1 = a1 * ddp * ddp
    t2 = a2 * ddp
    t3 = a4 * ddp * ddp
    t4 = a5 * ddp

    dinf = ((t1 + t2 + a3) * dds) + (t3 + t4 + a6)
    ff = b1 + b2 * (ddp - dds)
    n = dinf / df - ff * df + 1
    return n

# ----------------------------------------------------------------


class test_pm_remez(gr_unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_001_low_pass(self):
        gain = 1
        Fs = 1
        freq1 = 0.1
        freq2 = 0.2
        passband_ripple_db = 0.01
        stopband_atten_db = 60

        passband_dev = passband_ripple_to_dev(passband_ripple_db)
        stopband_dev = stopband_atten_to_dev(stopband_atten_db)
        desired_ampls = (gain, 0)
        (n, fo, ao, w) = remezord([freq1, freq2], desired_ampls,
                                  [passband_dev, stopband_dev], Fs)
        new_taps = filter.pm_remez(n + 2, fo, ao, w, "bandpass")

        known_taps = (-0.0008370135734511828, -0.0006622211673134374,
                      0.0008501079576365787, 0.003059609130249229,
                      0.003202235537205373, -0.001000899296974219,
                      -0.007589728680590891, -0.009790921118281865,
                      -0.001524210202628562, 0.014373535837200111,
                      0.02392881326993834, 0.011798133085019008,
                      -0.021954446348997188, -0.05293436740264934,
                      -0.04375787096766848, 0.028038890498420392,
                      0.14612655590172896, 0.25738578419108626,
                      0.302967004188747, 0.25738578419108626,
                      0.14612655590172896, 0.028038890498420392,
                      -0.04375787096766848, -0.05293436740264934,
                      -0.021954446348997188, 0.011798133085019008,
                      0.02392881326993834, 0.014373535837200111,
                      -0.001524210202628562, -0.009790921118281865,
                      -0.007589728680590891, -0.001000899296974219,
                      0.003202235537205373, 0.003059609130249229,
                      0.0008501079576365787, -0.0006622211673134374,
                      -0.0008370135734511828)

        self.assertFloatTuplesAlmostEqual(known_taps, new_taps, 5)

    def test_002_test_invalid_args(self):
        with self.assertRaises(ValueError):
            # two taps are too few
            filter.pm_remez(2, [0, 0.1], [1, 2e-4], [1, 0.25], "bandpass")
        with self.assertRaises(ValueError):
            # frequencies need to be ordered
            filter.pm_remez(200, [0.1, 0], [1, 2e-4], [1, 0.25], "bandpass")
        with self.assertRaises(ValueError):
            # bands, gains need to be of same length
            filter.pm_remez(200, [0, 0.1], [1, 2e-4, 2e-6], [1, 0.25], "bandpass")


if __name__ == '__main__':
    gr_unittest.run(test_pm_remez)



#!/usr/bin/env python
#
# Copyright 2005,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks


class test_single_pole_iir_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_ff_001(self):
        src_data = (0, 1000, 2000, 3000, 4000, 5000)
        expected_result = src_data
        src = blocks.vector_source_f(src_data)
        op = filter.single_pole_iir_filter_ff(1.0)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_ff_002(self):
        src_data = (0, 1000, 2000, 3000, 4000, 5000)
        expected_result = (
            0,
            125,
            359.375,
            689.453125,
            1103.271484,
            1590.36255)
        src = blocks.vector_source_f(src_data)
        op = filter.single_pole_iir_filter_ff(0.125)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 3)

    def test_ff_003(self):
        block_size = 2
        src_data = (0, 1000, 2000, 3000, 4000, 5000)
        expected_result = (0, 125, 250, 484.375, 718.75, 1048.828125)
        src = blocks.vector_source_f(src_data)
        s2p = blocks.stream_to_vector(gr.sizeof_float, block_size)
        op = filter.single_pole_iir_filter_ff(0.125, block_size)
        p2s = blocks.vector_to_stream(gr.sizeof_float, block_size)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, s2p, op, p2s, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 3)

    def test_cc_001(self):
        src_data = (
            0 + 0j,
            1000 + 1000j,
            2000 + 2000j,
            3000 + 3000j,
            4000 + 4000j,
            5000 + 5000j)
        expected_result = src_data
        src = blocks.vector_source_c(src_data)
        op = filter.single_pole_iir_filter_cc(1.0)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data)

    def test_cc_002(self):
        src_data = (complex(0, 0), complex(1000, -
                                           1000), complex(2000, -
                                                          2000), complex(3000, -
                                                                         3000), complex(4000, -
                                                                                        4000), complex(5000, -
                                                                                                       5000))
        expected_result = (complex(0, 0), complex(125, -
                                                  125), complex(359.375, -
                                                                359.375), complex(689.453125, -
                                                                                  689.453125), complex(1103.271484, -
                                                                                                       1103.271484), complex(1590.36255, -
                                                                                                                             1590.36255))
        src = blocks.vector_source_c(src_data)
        op = filter.single_pole_iir_filter_cc(0.125)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 3)

    def test_cc_003(self):
        block_size = 2
        src_data = (complex(0, 0), complex(1000, -
                                           1000), complex(2000, -
                                                          2000), complex(3000, -
                                                                         3000), complex(4000, -
                                                                                        4000), complex(5000, -
                                                                                                       5000))
        expected_result = (complex(0, 0), complex(125, -
                                                  125), complex(250, -
                                                                250), complex(484.375, -
                                                                              484.375), complex(718.75, -
                                                                                                718.75), complex(1048.828125, -
                                                                                                                 1048.828125))
        src = blocks.vector_source_c(src_data)
        s2p = blocks.stream_to_vector(gr.sizeof_gr_complex, block_size)
        op = filter.single_pole_iir_filter_cc(0.125, block_size)
        p2s = blocks.vector_to_stream(gr.sizeof_gr_complex, block_size)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, s2p, op, p2s, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 3)


if __name__ == '__main__':
    gr_unittest.run(test_single_pole_iir_filter)



#!/usr/bin/env python
#
# Copyright 2005,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import random


class test_packing(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [0x80, ]
        expected_results = [1, 0, 0, 0, 0, 0, 0, 0]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.packed_to_unpacked_bb(1, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_002(self):
        src_data = [0x80, ]
        expected_results = [0, 0, 0, 0, 0, 0, 0, 1]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.packed_to_unpacked_bb(1, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_003(self):
        src_data = [0x11, ]
        expected_results = [4, 2]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.packed_to_unpacked_bb(3, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_004(self):
        src_data = [0x11, ]
        expected_results = [0, 4]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.packed_to_unpacked_bb(3, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_005(self):
        src_data = [1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0]
        expected_results = [0x82, 0x5a]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_006(self):
        src_data = [0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0]
        expected_results = [0x82, 0x5a]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.unpacked_to_packed_bb(1, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_007(self):
        src_data = [4, 2, 0, 0, 0]
        expected_results = [0x11, ]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.unpacked_to_packed_bb(3, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_008(self):
        src_data = [0, 4, 2, 0, 0]
        expected_results = [0x11, ]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.unpacked_to_packed_bb(3, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_009(self):
        random.seed(0)
        src_data = []
        for i in range(202):
            src_data.append((random.randint(0, 255)))
        src_data = src_data
        expected_results = src_data

        src = blocks.vector_source_b(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_bb(3, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_bb(3, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results[0:201], dst.data())

    def test_010(self):
        random.seed(0)
        src_data = []
        for i in range(56):
            src_data.append((random.randint(0, 255)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_b(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_bb(7, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_bb(7, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results[0:201], dst.data())

    def test_011(self):
        random.seed(0)
        src_data = []
        for i in range(56):
            src_data.append((random.randint(0, 255)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_b(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_bb(7, gr.GR_LSB_FIRST)
        op2 = blocks.unpacked_to_packed_bb(7, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results[0:201], dst.data())

    # tests on shorts

    def test_100a(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**15, 2**15 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_s(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ss(1, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_ss(1, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_s()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_100b(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**15, 2**15 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_s(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ss(1, gr.GR_LSB_FIRST)
        op2 = blocks.unpacked_to_packed_ss(1, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_s()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_101a(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**15, 2**15 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_s(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ss(8, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_ss(8, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_s()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_101b(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**15, 2**15 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_s(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ss(8, gr.GR_LSB_FIRST)
        op2 = blocks.unpacked_to_packed_ss(8, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_s()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    # tests on ints

    def test_200a(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**31, 2**31 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_i(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ii(1, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_ii(1, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_200b(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**31, 2**31 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_i(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ii(1, gr.GR_LSB_FIRST)
        op2 = blocks.unpacked_to_packed_ii(1, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_201a(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**31, 2**31 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_i(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ii(8, gr.GR_MSB_FIRST)
        op2 = blocks.unpacked_to_packed_ii(8, gr.GR_MSB_FIRST)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())

    def test_201b(self):
        random.seed(0)
        src_data = []
        for i in range(100):
            src_data.append((random.randint(-2**31, 2**31 - 1)))
        src_data = src_data
        expected_results = src_data
        src = blocks.vector_source_i(tuple(src_data), False)
        op1 = blocks.packed_to_unpacked_ii(8, gr.GR_LSB_FIRST)
        op2 = blocks.unpacked_to_packed_ii(8, gr.GR_LSB_FIRST)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1, op2)
        self.tb.connect(op2, dst)
        self.tb.run()

        self.assertEqual(expected_results, dst.data())


if __name__ == '__main__':
    gr_unittest.run(test_packing)



#!/usr/bin/env python
# Copyright 2012-2016,2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import time
import random
import numpy

from gnuradio import gr
from gnuradio import gr_unittest
from gnuradio import digital
from gnuradio import blocks
import pmt


def make_tag(key, value, offset):
    """Create a gr.tag_t() from key, value, offset."""
    tag = gr.tag_t()
    tag.offset = offset
    tag.key = pmt.string_to_symbol(key)
    tag.value = pmt.to_pmt(value)
    return tag


class HeaderToMessageBlock(gr.sync_block):
    """
    Helps with testing the HPD. Receives a header, stores it, posts
    a predetermined message. forecast() is not currently working in
    Python, so use a local buffer to accumulate header data.
    """

    def __init__(self, itemsize, header_len, messages):
        gr.sync_block.__init__(
            self,
            name="HeaderToMessageBlock",
            in_sig=[itemsize],
            out_sig=[itemsize],
        )
        self.header_len = header_len
        self.message_port_register_out(pmt.intern('header_data'))
        self.messages = messages
        self.msg_count = 0
        self.buf = []

    def work(self, input_items, output_items):
        """Where the magic happens."""
        self.buf.extend(input_items[0])
        for _ in range(len(self.buf) // self.header_len):
            msg = self.messages[self.msg_count] or False
            self.message_port_pub(pmt.intern('header_data'), pmt.to_pmt(msg))
            self.msg_count += 1
            del self.buf[:self.header_len]
        output_items[0][:] = input_items[0][:]
        return len(input_items[0])


class qa_header_payload_demux (gr_unittest.TestCase):

    def setUp(self):
        """Runs before every test."""
        self.tb = gr.top_block()
        random.seed(0)

    def tearDown(self):
        """Runs after every test."""
        self.tb = None

    def connect_all_blocks(self,
                           data_src, trigger_src,
                           hpd,
                           mock_header_demod,
                           payload_sink, header_sink):
        """
        Connect the standard HPD test flowgraph
        """
        self.tb.connect(data_src, (hpd, 0))
        if trigger_src is not None:
            self.tb.connect(trigger_src, (hpd, 1))
        self.tb.connect((hpd, 0), mock_header_demod)
        self.tb.connect(mock_header_demod, header_sink)
        self.tb.msg_connect(
            mock_header_demod, 'header_data',
            hpd, 'header_data'
        )
        self.tb.connect((hpd, 1), payload_sink)

    def run_tb(
            self,
            payload_sink,
            payload_len,
            header_sink,
            header_len,
            timeout=30):
        """Execute self.tb"""
        stop_time = time.time() + timeout
        self.tb.start()
        while (len(payload_sink.data()) < payload_len or
                len(header_sink.data()) < header_len) and \
                time.time() < stop_time:
            time.sleep(.2)
        self.tb.stop()
        self.tb.wait()

    def test_001_t(self):
        """ Simplest possible test: put in zeros, then header,
        then payload, trigger signal, try to demux.
        The return signal from the header parser is faked via _post()
        Add in some tags for fun.
        """
        n_zeros = 1
        header = (1, 2, 3)
        payload = tuple(range(5, 20))
        data_signal = (0,) * n_zeros + header + payload
        trigger_signal = [0, ] * len(data_signal)
        trigger_signal[n_zeros] = 1
        # This is dropped:
        testtag1 = make_tag('tag1', 0, 0)
        # This goes on output 0, item 0:
        testtag2 = make_tag('tag2', 23, n_zeros)
        # This goes on output 0, item 2:
        testtag3 = make_tag('tag3', 42, n_zeros + len(header) - 1)
        # This goes on output 1, item 3:
        testtag4 = make_tag('tag4', 314, n_zeros + len(header) + 3)
        data_src = blocks.vector_source_f(
            data_signal,
            False,
            tags=(testtag1, testtag2, testtag3, testtag4)
        )
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            len(header), 1, 0, "frame_len", "detect", False, gr.sizeof_float
        )
        mock_header_demod = HeaderToMessageBlock(
            numpy.float32,
            len(header),
            [len(payload)]
        )
        # extra system port defined for you
        self.assertEqual(pmt.length(hpd.message_ports_in()), 2)
        payload_sink = blocks.vector_sink_f()
        header_sink = blocks.vector_sink_f()
        self.connect_all_blocks(
            data_src,
            trigger_src,
            hpd,
            mock_header_demod,
            payload_sink,
            header_sink)
        self.run_tb(payload_sink, len(payload), header_sink, len(header))
        self.assertEqual(header_sink.data(), list(header))
        self.assertEqual(payload_sink.data(), list(payload))
        ptags_header = []
        for tag in header_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_header.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_header = [
            {'key': 'tag2', 'offset': 0},
            {'key': 'tag3', 'offset': 2},
        ]
        self.assertEqual(expected_tags_header, ptags_header)
        ptags_payload = []
        for tag in payload_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_payload.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_payload = [
            {'key': 'frame_len', 'offset': 0},
            {'key': 'tag4', 'offset': 3},
        ]
        self.assertEqual(expected_tags_payload, ptags_payload)

    def test_001_t_tags(self):
        """ Like the previous test, but use a trigger tag instead of
        a trigger signal.
        """
        n_zeros = 1
        header = (1, 2, 3)
        payload = tuple(range(5, 20))
        data_signal = (0,) * n_zeros + header + payload
        # Trigger tag
        trigger_tag = make_tag('detect', True, n_zeros)
        # This is dropped:
        testtag1 = make_tag('tag1', 0, 0)
        # This goes on output 0, item 0:
        testtag2 = make_tag('tag2', 23, n_zeros)
        # This goes on output 0, item 2:
        testtag3 = make_tag('tag3', 42, n_zeros + len(header) - 1)
        # This goes on output 1, item 3:
        testtag4 = make_tag('tag4', 314, n_zeros + len(header) + 3)
        data_src = blocks.vector_source_f(
            data_signal,
            False,
            tags=(trigger_tag, testtag1, testtag2, testtag3, testtag4)
        )
        hpd = digital.header_payload_demux(
            len(header), 1, 0, "frame_len", "detect", False, gr.sizeof_float
        )
        # extra system port defined for you
        self.assertEqual(pmt.length(hpd.message_ports_in()), 2)
        header_sink = blocks.vector_sink_f()
        payload_sink = blocks.vector_sink_f()
        mock_header_demod = HeaderToMessageBlock(
            numpy.float32,
            len(header),
            [len(payload)]
        )
        self.connect_all_blocks(
            data_src,
            None,
            hpd,
            mock_header_demod,
            payload_sink,
            header_sink)
        self.run_tb(payload_sink, len(payload), header_sink, len(header))
        # Check results
        self.assertEqual(header_sink.data(), list(header))
        self.assertEqual(payload_sink.data(), list(payload))
        ptags_header = []
        for tag in header_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_header.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_header = [
            {'key': 'tag2', 'offset': 0},
            {'key': 'tag3', 'offset': 2},
        ]
        self.assertEqual(expected_tags_header, ptags_header)
        ptags_payload = []
        for tag in payload_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_payload.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_payload = [
            {'key': 'frame_len', 'offset': 0},
            {'key': 'tag4', 'offset': 3},
        ]
        self.assertEqual(expected_tags_payload, ptags_payload)

    def test_001_headerpadding(self):
        """ Like test 1, but with header padding. """
        n_zeros = 3
        header = [1, 2, 3]
        header_padding = 1
        payload = list(range(5, 20))
        data_signal = [0, ] * n_zeros + header + payload
        trigger_signal = [0] * len(data_signal)
        trigger_signal[n_zeros] = 1
        # This is dropped:
        testtag1 = make_tag('tag1', 0, 0)
        # This goes on output 0, item 0:
        testtag2 = make_tag('tag2', 23, n_zeros)
        # This goes on output 0, item 2:
        testtag3 = make_tag('tag3', 42, n_zeros + len(header) - 1)
        # This goes on output 1, item 3:
        testtag4 = make_tag('tag4', 314, n_zeros + len(header) + 3)
        data_src = blocks.vector_source_f(
            data_signal,
            False,
            tags=(testtag1, testtag2, testtag3, testtag4)
        )
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            len(header),
            1,  # Items per symbol
            0,  # Guard interval
            "frame_len",  # TSB tag key
            "detect",  # Trigger tag key
            False,  # No symbols please
            gr.sizeof_float,  # Item size
            "",  # Timing tag key
            1.0,  # Samp rate
            (),  # No special tags
            header_padding
        )
        mock_header_demod = HeaderToMessageBlock(
            numpy.float32,
            len(header),
            [len(payload)]
        )
        header_sink = blocks.vector_sink_f()
        payload_sink = blocks.vector_sink_f()
        self.connect_all_blocks(
            data_src,
            trigger_src,
            hpd,
            mock_header_demod,
            payload_sink,
            header_sink)
        self.run_tb(payload_sink, len(payload), header_sink, len(header) + 2)
        # Check values
        # Header now is padded:
        self.assertEqual(header_sink.data(), [0, ] + header + [payload[0], ])
        self.assertEqual(payload_sink.data(), payload)
        ptags_header = []
        for tag in header_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_header.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_header = [
            {'key': 'tag2', 'offset': 1},
            {'key': 'tag3', 'offset': 3},
        ]
        self.assertEqual(expected_tags_header, ptags_header)
        ptags_payload = []
        for tag in payload_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_payload.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_payload = [
            {'key': 'frame_len', 'offset': 0},
            {'key': 'tag4', 'offset': 3},
        ]
        self.assertEqual(expected_tags_payload, ptags_payload)

    def test_001_headerpadding_payload_offset(self):
        """ Like test 1, but with header padding + payload offset. """
        n_zeros = 3
        header = [1, 2, 3]
        header_padding = 1
        payload_offset = -1
        payload = list(range(5, 20))
        data_signal = [0, ] * n_zeros + header + payload + [0, ] * 100
        trigger_signal = [0] * len(data_signal)
        trigger_signal[n_zeros] = 1
        # This goes on output 1, item 3 + 1 (for payload offset)
        testtag4 = make_tag('tag4', 314, n_zeros + len(header) + 3)
        data_src = blocks.vector_source_f(
            data_signal,
            False,
            tags=(testtag4,)
        )
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            len(header),
            1,  # Items per symbol
            0,  # Guard interval
            "frame_len",  # TSB tag key
            "detect",  # Trigger tag key
            False,  # No symbols please
            gr.sizeof_float,  # Item size
            "",  # Timing tag key
            1.0,  # Samp rate
            (),  # No special tags
            header_padding
        )
        # extra system port defined for you
        self.assertEqual(pmt.length(hpd.message_ports_in()), 2)
        header_sink = blocks.vector_sink_f()
        payload_sink = blocks.vector_sink_f()
        self.tb.connect(data_src, (hpd, 0))
        self.tb.connect(trigger_src, (hpd, 1))
        self.tb.connect((hpd, 0), header_sink)
        self.tb.connect((hpd, 1), payload_sink)
        self.tb.start()
        time.sleep(.2)  # Need this, otherwise, the next message is ignored
        hpd.to_basic_block()._post(pmt.intern('header_data'), pmt.to_pmt(
            {'frame_len': len(payload), 'payload_offset': payload_offset}))
        while len(payload_sink.data()) < len(payload):
            time.sleep(.2)
        self.tb.stop()
        self.tb.wait()
        # Header is now padded:
        self.assertEqual(header_sink.data(), [0, ] + header + [payload[0], ])
        # Payload is now offset:
        self.assertEqual(payload_sink.data(), data_signal[n_zeros + len(
            header) + payload_offset:n_zeros + len(header) + payload_offset + len(payload)])
        ptags_payload = {}
        for tag in payload_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_payload[ptag.key] = ptag.offset
        expected_tags_payload = {
            'frame_len': 0,
            'payload_offset': 0,
            'tag4': 3 - payload_offset,
        }
        self.assertEqual(expected_tags_payload, ptags_payload)

    def test_002_symbols(self):
        """
        Same as before, but operate on symbols
        """
        n_zeros = 1
        items_per_symbol = 3
        gi = 1
        n_symbols = 4
        header = (1, 2, 3)
        payload = (1, 2, 3)
        data_signal = (0,) * n_zeros + (0,) + header + \
            ((0,) + payload) * n_symbols
        trigger_signal = [0, ] * len(data_signal)
        trigger_signal[n_zeros] = 1
        # This is dropped:
        testtag1 = make_tag('tag1', 0, 0)
        # This goes on output 0, item 0 (from the GI)
        testtag2 = make_tag('tag2', 23, n_zeros)
        # This goes on output 0, item 0 (middle of the header symbol)
        testtag3 = make_tag('tag3', 42, n_zeros + gi + 1)
        # This goes on output 1, item 1 (middle of the first payload symbol)
        testtag4 = make_tag('tag4', 314, n_zeros +
                            (gi + items_per_symbol) * 2 + 1)
        data_src = blocks.vector_source_f(
            data_signal, False, tags=(
                testtag1, testtag2, testtag3, testtag4))
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            len(header) // items_per_symbol,  # Header length (in symbols)
            items_per_symbol,                # Items per symbols
            gi,                              # Items per guard time
            "frame_len",                     # Frame length tag key
            "detect",                        # Trigger tag key
            True,                            # Output symbols (not items)
            gr.sizeof_float                  # Bytes per item
        )
        # extra system port defined for you
        self.assertEqual(pmt.length(hpd.message_ports_in()), 2)
        header_sink = blocks.vector_sink_f(items_per_symbol)
        payload_sink = blocks.vector_sink_f(items_per_symbol)
        self.tb.connect(data_src, (hpd, 0))
        self.tb.connect(trigger_src, (hpd, 1))
        self.tb.connect((hpd, 0), header_sink)
        self.tb.connect((hpd, 1), payload_sink)
        self.tb.start()
        time.sleep(.2)  # Need this, otherwise, the next message is ignored
        hpd.to_basic_block()._post(
            pmt.intern('header_data'),
            pmt.from_long(n_symbols)
        )
        while len(payload_sink.data()) < len(payload) * n_symbols:
            time.sleep(.2)
        self.tb.stop()
        self.tb.wait()
        self.assertEqual(header_sink.data(), list(header))
        self.assertEqual(payload_sink.data(), list(payload * n_symbols))
        ptags_header = []
        for tag in header_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_header.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_header = [
            {'key': 'tag2', 'offset': 0},
            {'key': 'tag3', 'offset': 0},
        ]
        self.assertEqual(expected_tags_header, ptags_header)
        ptags_payload = []
        for tag in payload_sink.tags():
            ptag = gr.tag_to_python(tag)
            ptags_payload.append({'key': ptag.key, 'offset': ptag.offset})
        expected_tags_payload = [
            {'key': 'frame_len', 'offset': 0},
            {'key': 'tag4', 'offset': 1},
        ]
        self.assertEqual(expected_tags_payload, ptags_payload)

    def test_003_t(self):
        """
        Like test 1, but twice, plus one fail
        """
        # Tx Data
        n_zeros = 5
        header = [1, 2, 3]
        header_fail = [-1, -2, -4]  # Contents don't really matter
        payload1 = list(range(5, 20))
        payload2 = [42, ]
        sampling_rate = 2
        data_signal = [0, ] * n_zeros + header + payload1
        trigger_signal = [0, ] * len(data_signal) * 2
        trigger_signal[n_zeros] = 1
        trigger_signal[len(data_signal)] = 1
        trigger_signal[len(data_signal) + len(header_fail) + n_zeros] = 1
        print("Triggers at: {0} {1} {2}".format(
            n_zeros,
            len(data_signal),
            len(data_signal) + len(header_fail) + n_zeros))
        tx_signal = data_signal + \
            header_fail + [0, ] * n_zeros + \
            header + payload2 + [0, ] * 1000
        # Timing tag: This is preserved and updated:
        timing_tag = make_tag('rx_time', (0, 0), 0)
        # Rx freq tags:
        rx_freq_tag1 = make_tag('rx_freq', 1.0, 0)
        rx_freq_tag2 = make_tag('rx_freq', 1.5, 29)
        rx_freq_tag3 = make_tag('rx_freq', 2.0, 30)
        # Flow graph
        data_src = blocks.vector_source_f(
            tx_signal, False,
            tags=(timing_tag, rx_freq_tag1, rx_freq_tag2, rx_freq_tag3)
        )
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            header_len=len(header),
            items_per_symbol=1,
            guard_interval=0,
            length_tag_key="frame_len",
            trigger_tag_key="detect",
            output_symbols=False,
            itemsize=gr.sizeof_float,
            timing_tag_key='rx_time',
            samp_rate=sampling_rate,
            special_tags=('rx_freq',),
        )
        # extra system port defined for you
        self.assertEqual(pmt.length(hpd.message_ports_in()), 2)
        header_sink = blocks.vector_sink_f()
        payload_sink = blocks.vector_sink_f()
        self.tb.connect(data_src, (hpd, 0))
        self.tb.connect(trigger_src, (hpd, 1))
        self.tb.connect((hpd, 0), header_sink)
        self.tb.connect((hpd, 1), payload_sink)
        self.tb.start()
        time.sleep(.2)  # Need this, otherwise, the next message is ignored
        hpd.to_basic_block()._post(
            pmt.intern('header_data'),
            pmt.from_long(len(payload1))
        )
        while len(payload_sink.data()) < len(payload1):
            time.sleep(.2)
        hpd.to_basic_block()._post(
            pmt.intern('header_data'),
            pmt.PMT_F
        )
        # This next command is a bit of a showstopper, but there's no condition to check upon
        # to see if the previous msg handling is finished
        time.sleep(.7)
        hpd.to_basic_block()._post(
            pmt.intern('header_data'),
            pmt.from_long(len(payload2))
        )
        while len(payload_sink.data()) < len(payload1) + len(payload2):
            time.sleep(.2)
        self.tb.stop()
        self.tb.wait()
        # Signal description:
        # 0:  5 zeros
        # 5:  header 1
        # 8:  payload 1 (length: 15)
        # 23: header 2 (fail)
        # 26: 5 zeros
        # 31: header 3
        # 34: payload 2 (length 1)
        # 35: 1000 zeros
        self.assertEqual(
            header_sink.data(), list(
                header + header_fail + header))
        self.assertEqual(payload_sink.data(), payload1 + payload2)
        tags_payload = [gr.tag_to_python(x) for x in payload_sink.tags()]
        tags_payload = sorted([(x.offset, x.key, x.value)
                               for x in tags_payload])
        tags_expected_payload = [
            (0, 'frame_len', len(payload1)),
            (len(payload1), 'frame_len', len(payload2)),
        ]
        tags_header = [gr.tag_to_python(x) for x in header_sink.tags()]
        tags_header = sorted([(x.offset, x.key, x.value) for x in tags_header])
        tags_expected_header = [
            (0, 'rx_freq', 1.0),
            # Hard coded time value :( Is n_zeros/sampling_rate
            (0, 'rx_time', (2, 0.5)),
            (len(header), 'rx_freq', 1.0),
            # Hard coded time value :(. See above.
            (len(header), 'rx_time', (11, .5)),
            (2 * len(header), 'rx_freq', 2.0),
            # Hard coded time value :(. See above.
            (2 * len(header), 'rx_time', (15, .5)),
        ]
        self.assertEqual(tags_header, tags_expected_header)
        self.assertEqual(tags_payload, tags_expected_payload)

    def test_004_fuzz(self):
        """
        Long random test
        """
        def create_signal(
                n_bursts,
                header_len,
                max_gap,
                max_burstsize,
                fail_rate,
        ):
            signal = []
            indexes = []
            burst_sizes = []
            total_payload_len = 0
            for _ in range(n_bursts):
                gap_size = random.randint(0, max_gap)
                signal += [0] * gap_size
                is_failure = random.random() < fail_rate
                if not is_failure:
                    burst_size = random.randint(0, max_burstsize)
                else:
                    burst_size = 0
                total_payload_len += burst_size
                indexes += [len(signal)]
                signal += [1] * header_len
                signal += [2] * burst_size
                burst_sizes += [burst_size]
            return (signal, indexes, total_payload_len, burst_sizes)

        def indexes_to_triggers(indexes, signal_len):
            """
            Convert indexes to a mix of trigger signals and tags
            """
            trigger_signal = [0] * signal_len
            trigger_tags = []
            for index in indexes:
                if random.random() > 0.5:
                    trigger_signal[index] = 1
                else:
                    trigger_tags += [make_tag('detect', True, index)]
            return (trigger_signal, trigger_tags)
        ### Go, go, go
        # Uncomment this if you want true randomness -- good for actual fuzzing
        # The divide-by-20 means we'll usually get the same random seed
        # between the first run and the XML run.
        # random_seed = int(time.time() / 20)
        # random.seed(random_seed)
        # print("Random seed: {0}".format(random_seed))
        n_bursts = 400
        header_len = 5
        max_gap = 50
        max_burstsize = 100
        fail_rate = 0.05
        signal, indexes, total_payload_len, burst_sizes = create_signal(
            n_bursts, header_len, max_gap, max_burstsize, fail_rate
        )
        trigger_signal, trigger_tags = indexes_to_triggers(
            indexes, len(signal))
        # Flow graph
        data_src = blocks.vector_source_f(
            signal, False,
            tags=trigger_tags
        )
        trigger_src = blocks.vector_source_b(trigger_signal, False)
        hpd = digital.header_payload_demux(
            header_len=header_len,
            items_per_symbol=1,
            guard_interval=0,
            length_tag_key="frame_len",
            trigger_tag_key="detect",
            output_symbols=False,
            itemsize=gr.sizeof_float,
            timing_tag_key='rx_time',
            samp_rate=1.0,
            special_tags=('rx_freq',),
        )
        mock_header_demod = HeaderToMessageBlock(
            numpy.float32,
            header_len,
            burst_sizes
        )
        header_sink = blocks.vector_sink_f()
        payload_sink = blocks.vector_sink_f()
        self.connect_all_blocks(
            data_src,
            trigger_src,
            hpd,
            mock_header_demod,
            payload_sink,
            header_sink)
        self.run_tb(
            payload_sink,
            total_payload_len,
            header_sink,
            header_len *
            n_bursts)
        self.assertEqual(header_sink.data(), list([1] * header_len * n_bursts))
        self.assertEqual(payload_sink.data(), list([2] * total_payload_len))


if __name__ == '__main__':
    gr_unittest.run(qa_header_payload_demux)



#!/usr/bin/env python
#
# Copyright 2008,2010,2013,2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import os
import tempfile
import unittest
import wave
from os.path import getsize

g_in_file = os.path.join(os.getenv("srcdir"), "test_16bit_1chunk.wav")
g_in_file_normal = os.path.join(
    os.getenv("srcdir"),
    "test_16bit_1chunk_normal.wav")
g_in_file_mp3 = os.path.join(os.getenv("srcdir"), "test_no_info_frame.mp3")
g_extra_header_offset = 36
g_extra_header_len = 22


class test_wavefile(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_checkwavread(self):
        wf = blocks.wavfile_source(g_in_file)
        self.assertEqual(wf.sample_rate(), 8000)

    def test_002_checkwavcopy(self):
        infile = g_in_file
        outfile = "test_out.wav"

        wf_in = blocks.wavfile_source(infile)
        wf_out = blocks.wavfile_sink(outfile,
                                     wf_in.channels(),
                                     wf_in.sample_rate(),
                                     blocks.FORMAT_WAV,
                                     blocks.FORMAT_PCM_16)
        self.tb.connect(wf_in, wf_out)
        self.tb.run()
        wf_out.close()

        # Test file validity.
        try:
            with wave.open(infile, 'rb') as f:
                pass
            with wave.open(outfile, 'rb') as f:
                pass
        except BaseException:
            raise AssertionError('Invalid WAV file')

        # we're losing all extra header chunks
        self.assertEqual(
            getsize(infile) -
            g_extra_header_len,
            getsize(outfile))

        with open(infile, 'rb') as f:
            in_data = bytearray(f.read())
        with open(outfile, 'rb') as f:
            out_data = bytearray(f.read())

        os.remove(outfile)

        # Ignore size field:
        in_data[4:8] = b'\x00\x00\x00\x00'
        out_data[4:8] = b'\x00\x00\x00\x00'

        # cut extra header chunks from input file
        self.assertEqual(in_data[:g_extra_header_offset] +
                         in_data[g_extra_header_offset +
                                 g_extra_header_len:], out_data)

    def test_003_checkwav_append_copy(self):
        infile = g_in_file_normal
        outfile = "test_out_append.wav"

        # 1. Copy input to output
        from shutil import copyfile
        copyfile(infile, outfile)

        # 2. append copy
        wf_in = blocks.wavfile_source(infile)
        wf_out = blocks.wavfile_sink(outfile,
                                     wf_in.channels(),
                                     wf_in.sample_rate(),
                                     blocks.FORMAT_WAV,
                                     blocks.FORMAT_PCM_16,
                                     True)
        self.tb.connect(wf_in, wf_out)
        self.tb.run()
        wf_out.close()

        # 3. append halved copy
        self.tb.disconnect_all()
        wf_in = blocks.wavfile_source(infile)
        halver = blocks.multiply_const_ff(0.5)
        wf_out = blocks.wavfile_sink(outfile,
                                     wf_in.channels(),
                                     wf_in.sample_rate(),
                                     blocks.FORMAT_WAV,
                                     blocks.FORMAT_PCM_16,
                                     True)
        self.tb.connect(wf_in, halver, wf_out)
        self.tb.run()
        wf_out.close()

        # Test file validity and read data.
        try:
            # In
            with wave.open(infile, 'rb') as w_in:
                in_params = w_in.getparams()
                data_in = wav_read_frames(w_in)
            # Out
            with wave.open(outfile, 'rb') as w_out:
                out_params = w_out.getparams()
                data_out = wav_read_frames(w_out)
        except BaseException:
            raise AssertionError('Invalid WAV file')

        # Params must be equal except in size:
        expected_params = in_params._replace(nframes=3 * in_params.nframes)
        self.assertEqual(out_params, expected_params)

        # Part 1
        self.assertEqual(data_in, data_out[:len(data_in)])

        # Part 2
        self.assertEqual(data_in, data_out[len(data_in):2 * len(data_in)])

        # Part 3
        data_in_halved = [int(round(d / 2)) for d in data_in]
        self.assertEqual(data_in_halved, data_out[2 * len(data_in):])

        os.remove(outfile)

    def test_003_checkwav_append_non_existent_should_error(self):
        outfile = "no_file.wav"

        with self.assertRaisesRegex(RuntimeError, "Can't open WAV file."):
            blocks.wavfile_sink(
                outfile,
                1,
                44100,
                blocks.FORMAT_WAV,
                blocks.FORMAT_PCM_16,
                True)

        os.remove(outfile)

    def test_004_automatic_reopen_after_stop(self):
        infile = g_in_file_normal
        outfile = "test_out_append.wav"

        # 1. Copy input to output
        wf_in1 = blocks.wavfile_source(infile)
        wf_out = blocks.wavfile_sink(outfile,
                                     wf_in1.channels(),
                                     wf_in1.sample_rate(),
                                     blocks.FORMAT_WAV,
                                     blocks.FORMAT_PCM_16,
                                     False)
        self.tb.connect(wf_in1, wf_out)
        self.tb.run()

        # 2. lock and reconfigure to append the same copy again
        self.tb.lock()
        self.tb.disconnect(wf_in1, wf_out)
        wf_in2 = blocks.wavfile_source(infile)
        self.tb.connect(wf_in2, wf_out)
        self.tb.unlock()
        self.tb.run()

        wf_out.close()

        # Test file validity and read data.
        try:
            # In
            with wave.open(infile, 'rb') as w_in:
                in_params = w_in.getparams()
                data_in = wav_read_frames(w_in)
            # Out
            with wave.open(outfile, 'rb') as w_out:
                out_params = w_out.getparams()
                data_out = wav_read_frames(w_out)
        except BaseException:
            raise AssertionError('Invalid WAV file')

        # Params must be equal except in size:
        expected_params = in_params._replace(nframes=2 * in_params.nframes)
        self.assertEqual(out_params, expected_params)

        # Part 1
        self.assertEqual(data_in, data_out[:len(data_in)])

        # Part 2
        self.assertEqual(data_in, data_out[len(data_in):2 * len(data_in)])

        os.remove(outfile)

    def test_read_wav(self):
        expected_len = 4

        src = blocks.wavfile_source(g_in_file_normal)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, dst)
        self.tb.run()

        result = dst.data()

        self.assertEqual(len(result), expected_len)

    def test_read_wav_repeat(self):
        expected_len = 4
        repeats = 100

        src = blocks.wavfile_source(g_in_file_normal, repeat=True)
        head = blocks.head(gr.sizeof_float, expected_len * repeats)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, head, dst)
        self.tb.run()

        result = dst.data()

        self.assertEqual(len(result), expected_len * repeats)
        for i in range(1, repeats):
            offset = i * expected_len
            self.assertEqual(result[offset:offset + expected_len], result[0:expected_len])

    def test_read_wav_1024(self):
        for expected_len in range(1024, 32768, 1024):
            with tempfile.NamedTemporaryFile(suffix=".wav", delete=False) as tf:
                with wave.open(tf.name, "w") as wf:
                    wf.setnchannels(1)
                    wf.setsampwidth(2)
                    wf.setframerate(8000)
                    wf.writeframes(bytes([0, 0] * expected_len))

                self.tb = gr.top_block()
                src = blocks.wavfile_source(tf.name)
                dst = blocks.vector_sink_f()
                self.tb.connect(src, dst)
                self.tb.run()

                result = dst.data()

                del src
                del self.tb

                self.assertEqual(len(result), expected_len)

            os.unlink(tf.name)

    @unittest.skip("skipped due to bug in libsndfile < 1.2.1")
    def test_read_mp3(self):
        expected_len = 2304

        src = blocks.wavfile_source(g_in_file_mp3)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, dst)
        self.tb.run()

        result = dst.data()

        self.assertEqual(len(result), expected_len)

    @unittest.skip("skipped due to bug in libsndfile < 1.2.1")
    def test_read_mp3_repeat(self):
        expected_len = 2304
        repeats = 5

        src = blocks.wavfile_source(g_in_file_mp3, repeat=True)
        head = blocks.head(gr.sizeof_float, expected_len * repeats)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, head, dst)
        self.tb.run()

        result = dst.data()

        self.assertEqual(len(result), expected_len * repeats)
        for i in range(1, repeats):
            offset = i * expected_len
            self.assertEqual(result[offset:offset + expected_len], result[0:expected_len])


def wav_read_frames(w):
    import struct
    # grouper from itertools recipes.
    def grouper(iterable, n): return list(zip(* ([iter(iterable)] * n)))
    assert w.getsampwidth() == 2  # Assume 16 bits
    return [
        struct.unpack('h', bytes(frame_g))[0]
        for frame_g in grouper(w.readframes(w.getnframes()), 2)
    ]


if __name__ == '__main__':
    gr_unittest.run(test_wavefile)



import argparse
import os
from gnuradio.bindtool import BindingGenerator
import pathlib

parser = argparse.ArgumentParser(
    description='Bind a GR header file from the generated json')
parser.add_argument('pathnames', type=str, nargs='+',
                    help='Json files to bind')

args = parser.parse_args()

bg = BindingGenerator()

for p in args.pathnames:
    bg.bind_from_json(p)



# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import sys
from PyQt5 import Qt, QtCore
from math import sin, cos, pi
import pyqtgraph as pg
from numpy import zeros
from numpy import float64 as Float
from numpy import vectorize
from numpy import delete


class PzPlot(pg.PlotWidget):

    def __init__(self, *args):
        pg.PlotWidget.__init__(self, *args)

        # Set Global pyqtgraph options
        # Default foreground color for text, lines, axes, etc.
        pg.setConfigOption('foreground', 'k')
        # Default background for GraphicsView.
        pg.setConfigOption('background', None)
        # Draw lines with smooth edges at the cost of reduced performance.
        pg.setConfigOptions(antialias=True)

        self.ymax = 0
        self.xmax = 0
        self.ymin = 0
        self.xmin = 0
        self.setBackground(Qt.QBrush(Qt.Qt.darkCyan))

        self.setRange(xRange=[-3, 3], yRange=[-2, 2])

        axis = self.getAxis('bottom')
        axis.setStyle(tickLength=-10)
        axis.setPen(Qt.QPen(Qt.Qt.white, 1.025, Qt.Qt.DotLine))

        axis = self.getAxis('left')
        axis.setStyle(tickLength=-10)
        axis.setPen(Qt.QPen(Qt.Qt.white, 1.025, Qt.Qt.DotLine))

        self.plotItem.showGrid(x=True, y=True, alpha=100)

        self.drawUnitcircle()

    def drawUnitcircle(self):
        radius = 1.0
        steps = 1024

        angleStep = 2 * pi / steps
        x = [sin(a * angleStep) * radius for a in range(0, steps)]
        y = [cos(a * angleStep) * radius for a in range(0, steps)]

        curve = self.plot(title="Unit Circle")
        curve.setPen(Qt.QPen(Qt.Qt.gray, 0.025, Qt.Qt.DotLine))
        curve.setData(x, y)

    def insertZeros(self, roots):
        self.removeallCurves()
        if len(roots):
            self.__insertZero(Qt.Qt.blue, roots.real, roots.imag)
            self.ymax = max(roots.imag)
            self.ymin = min(roots.imag)
            self.xmax = max(roots.real)
            self.xmin = min(roots.real)
            # To make the plot look good.
            if self.xmax <= 1.3:
                self.xmax = 2
            if self.xmin >= -1.3:
                self.xmin = -2
            if self.ymax <= 1:
                self.ymax = 1.5
            if self.ymin >= -1:
                self.ymin = -1.5

            self.setRange(xRange=[self.xmin, self.xmax],
                          yRange=[self.ymin, self.ymax])

    def insertPoles(self, roots):
        if len(roots):
            self.__insertPole(Qt.Qt.black, roots.real, roots.imag)
            ymax = max(roots.imag)
            ymax = max(ymax, self.ymax)
            ymin = min(roots.imag)
            ymin = min(ymin, self.ymin)
            xmax = max(roots.real)
            xmax = max(xmax, self.xmax)
            xmin = min(roots.real)
            xmin = min(xmin, self.xmin)

            # To make the plot look good.
            if xmax <= 1.3:
                xmax = 2
            else:
                xmax = 1.2 * xmax

            if xmin >= -1.3:
                xmin = -2
            else:
                xmin = 1.2 * xmin

            if ymax <= 1:
                ymax = 1.5
            else:
                ymax = 1.2 * ymax

            if ymin >= -1:
                ymin = -1.5
            else:
                ymin = 1.2 * ymin

            self.setRange(xRange=[xmin, xmax], yRange=[ymin, ymax])
            self.drawUnitcircle()
            self.replot()

    def __insertZero(self, color, px, py):
        curve = self.plot(name="Zero")
        curve.setPen(None)
        curve.setSymbol('o')
        curve.setSymbolPen('b')
        curve.setSymbolBrush(Qt.QBrush(Qt.Qt.gray))
        curve.setSymbolSize(10)
        curve.setData(px, py)

    def __insertPole(self, color, px, py):
        curve = self.plot(name="Pole")
        curve.setPen(None)
        curve.setSymbol('x')
        curve.setSymbolPen('b')
        curve.setSymbolBrush(Qt.QBrush(Qt.Qt.gray))
        curve.setSymbolSize(10)
        curve.setData(px, py)

    def removeallCurves(self):
        # TODO for curve in self.itemList():
        # if isinstance(curve, Qwt.QwtPlotCurve):
        # curve.detach()
        self.replot()


class CanvasPicker(Qt.QObject):
    curveChanged = QtCore.pyqtSignal(tuple)
    mouseposChanged = QtCore.pyqtSignal(tuple)

    def __init__(self, plot):
        Qt.QObject.__init__(self, plot)
        self.__selectedCurve = None
        self.__selectedPoint = -1
        self.__selectedcPoint = -1
        self.__addedZero = -1
        self.__addedcZero = -1
        self.changeConjugate = False
        self.enableZeroadd = False
        self.enablePoleadd = False
        self.enablepzDelete = False
        self.iir = False
        self.__plot = plot

        ''' TODO
        canvas = plot.canvas()
        canvas.installEventFilter(self)

        # We want the focus, but no focus rect.
        # The selected point will be highlighted instead.
        canvas.setFocusPolicy(Qt.Qt.StrongFocus)
        canvas.setCursor(Qt.Qt.PointingHandCursor)
        canvas.setFocusIndicator(Qwt.QwtPlotCanvas.ItemFocusIndicator)
        canvas.setFocus()

        canvas.setWhatsThis(
            'All points can be moved using the left mouse button '
            'or with these keys:\n\n'
            '- Up: Select next curve\n'
            '- Down: Select previous curve\n'
            '- Left, "-": Select next point\n'
            '- Right, "+": Select previous point\n'
            '- 7, 8, 9, 4, 6, 1, 2, 3: Move selected point'
            )

        self.__shiftCurveCursor(True)
        '''

    def event(self, event):
        if event.type() == Qt.QEvent.User:
            self.__showCursor(True)
            return True
        try:
            return Qt.QObject.event(event)
        except TypeError:
            return False

    def set_conjugate(self):
        self.changeConjugate = not(self.changeConjugate)

    def set_iir(self, val=True):
        self.iir = val

    def add_zero(self):
        self.enableZeroadd = not(self.enableZeroadd)

    def add_pole(self):
        # Adding pole support only for IIR.
        if self.iir:
            self.enablePoleadd = not(self.enablePoleadd)

    def delete_pz(self):
        self.enablepzDelete = not(self.enablepzDelete)

    def eventFilter(self, object, event):

        if event.type() == Qt.QEvent.FocusIn:
            self.__showCursor(True)
        if event.type() == Qt.QEvent.FocusOut:
            self.__showCursor(False)

        if event.type() == Qt.QEvent.Paint:
            Qt.QApplication.postEvent(
                self, Qt.QEvent(Qt.QEvent.User))
        elif event.type() == Qt.QEvent.MouseButtonPress:
            if self.enableZeroadd or self.enablePoleadd:
                self.__drawAddedzero_pole(True, event.pos())
            else:
                self.__select(event.pos())
            return True
        elif event.type() == Qt.QEvent.MouseMove:
            curve = self.__selectedCurve
            if curve:
                tp = (self.__plot.invTransform(curve.xAxis(), event.pos().x()),
                      self.__plot.invTransform(curve.xAxis(), event.pos().y()))
                self.mouseposChanged.emit(tp)
            self.__move(event.pos())
            return True

        if event.type() == Qt.QEvent.KeyPress:
            delta = 5
            key = event.key()
            if key == Qt.Qt.Key_Up:
                self.__shiftCurveCursor(True)
                return True
            elif key == Qt.Qt.Key_Down:
                self.__shiftCurveCursor(False)
                return True
            elif key == Qt.Qt.Key_Right or key == Qt.Qt.Key_Plus:
                if self.__selectedCurve:
                    self.__shiftPointCursor(True)
                else:
                    self.__shiftCurveCursor(True)
                return True
            elif key == Qt.Qt.Key_Left or key == Qt.Qt.Key_Minus:
                if self.__selectedCurve:
                    self.__shiftPointCursor(False)
                else:
                    self.__shiftCurveCursor(True)
                return True

            if key == Qt.Qt.Key_1:
                self.__moveBy(-delta, delta)
            elif key == Qt.Qt.Key_2:
                self.__moveBy(0, delta)
            elif key == Qt.Qt.Key_3:
                self.__moveBy(delta, delta)
            elif key == Qt.Qt.Key_4:
                self.__moveBy(-delta, 0)
            elif key == Qt.Qt.Key_6:
                self.__moveBy(delta, 0)
            elif key == Qt.Qt.Key_7:
                self.__moveBy(-delta, -delta)
            elif key == Qt.Qt.Key_8:
                self.__moveBy(0, -delta)
            elif key == Qt.Qt.Key_9:
                self.__moveBy(delta, -delta)

        return Qwt.QwtPlot.eventFilter(self, object, event)

    def __select(self, pos):
        found, distance, point = None, 1e100, -1

        for curve in self.__plot.itemList():
            if isinstance(curve, Qwt.QwtPlotCurve):
                if curve.symbol().style() != Qwt.QwtSymbol.NoSymbol:
                    i, d = curve.closestPoint(pos)
                    if d < distance:
                        found = curve
                        point = i
                        distance = d

        self.__showCursor(False)
        self.__selectedCurve = None
        self.__selectedPoint = -1
        self.__selectedcPoint = -1

        if found and distance < 10:
            self.__selectedCurve = found
            self.__selectedPoint = point
            # Search for conjugate point if enabled.
            if self.changeConjugate:
                j = self.__searchConjugate(found.x(point), found.y(point))
                self.__selectedcPoint = j
            # Delete zero/pole if enabled.
            if self.enablepzDelete:
                self.__deleteZero()
            self.__showCursor(True)
        else:
            self.mouseposChanged.emit((None, None))

    def __deleteZero(self):
        curve = self.__selectedCurve
        xData = zeros(curve.dataSize(), Float)
        yData = zeros(curve.dataSize(), Float)

        for i in range(curve.dataSize()):
            xData[i] = curve.x(i)
            yData[i] = curve.y(i)

        if(self.__selectedPoint != -1):
            xData = delete(xData, self.__selectedPoint)
            yData = delete(yData, self.__selectedPoint)
        # One less to accommodate previous delete.
        if(self.__selectedcPoint != -1):
            xData = delete(xData, self.__selectedcPoint - 1)
            yData = delete(yData, self.__selectedcPoint - 1)

        curve.setData(xData, yData)
        self.__plot.replot()
        px = []
        py = []
        for c in self.__plot.itemList():
            if isinstance(c, Qwt.QwtPlotCurve):
                px.append([c.x(i) for i in range(c.dataSize())])
                py.append([c.y(i) for i in range(c.dataSize())])
        tp = (vectorize(complex)(px[0], py[0]),
              vectorize(complex)(px[1], py[1]))
        self.curveChanged.emit(tp)

    def __moveBy(self, dx, dy):
        if dx == 0 and dy == 0:
            return

        curve = self.__selectedCurve
        if not curve:
            return

        x = self.__plot.transform(
            curve.xAxis(), curve.x(self.__selectedPoint)) + dx
        y = self.__plot.transform(
            curve.yAxis(), curve.y(self.__selectedPoint)) + dy
        self.__move(Qt.QPoint(x, y))

    def __move(self, pos):
        curve = self.__selectedCurve
        if not curve:
            return

        xData = zeros(curve.dataSize(), Float)
        yData = zeros(curve.dataSize(), Float)

        # Poles made unmovable when FIR design is active.
        move_enable = True
        if not self.iir:
            if self.__selectedCurve.symbol().style() == Qwt.QwtSymbol.XCross:
                move_enable = False
        if move_enable:
            for i in range(curve.dataSize()):
                if i == self.__selectedPoint:
                    xData[i] = self.__plot.invTransform(curve.xAxis(), pos.x())
                    yData[i] = self.__plot.invTransform(curve.yAxis(), pos.y())
                    if(self.__selectedcPoint != -1):
                        xData[self.__selectedcPoint] = xData[i]
                        yData[self.__selectedcPoint] = -yData[i]
                elif i != self.__selectedcPoint:
                    xData[i] = curve.x(i)
                    yData[i] = curve.y(i)
            curve.setData(xData, yData)
            self.__plot.replot()
            px = []
            py = []
            for c in self.__plot.itemList():
                if isinstance(c, Qwt.QwtPlotCurve):
                    px.append([c.x(i) for i in range(c.dataSize())])
                    py.append([c.y(i) for i in range(c.dataSize())])
            tp = (vectorize(complex)(px[0], py[0]),
                  vectorize(complex)(px[1], py[1]))
            self.curveChanged.emit(tp)
            self.__showCursor(True)

    def __searchConjugate(self, x, y):
        curve = self.__selectedCurve
        for i in range(curve.dataSize()):
            if (round(curve.x(i), 8) == round(x, 8) and round(curve.y(i), 8) == -round(y, 8)):
                if (y != 0):
                    return i
        return -1

    def __drawAddedzero_pole(self, showIt, pos):
        editcurve = ''
        for curve in self.__plot.itemList():
            if isinstance(curve, Qwt.QwtPlotCurve):
                if self.enableZeroadd:
                    if curve.symbol().style() == Qwt.QwtSymbol.Ellipse:
                        editcurve = curve
                if self.enablePoleadd:
                    if curve.symbol().style() == Qwt.QwtSymbol.XCross:
                        editcurve = curve
        if not editcurve:
            return

        if self.changeConjugate:
            extrapoints = 2
        else:
            extrapoints = 1

        xData = zeros(editcurve.dataSize() + extrapoints, Float)
        yData = zeros(editcurve.dataSize() + extrapoints, Float)

        for i in range(editcurve.dataSize()):
            xData[i] = editcurve.x(i)
            yData[i] = editcurve.y(i)
        xData[i + 1] = self.__plot.invTransform(editcurve.xAxis(), pos.x())
        yData[i + 1] = self.__plot.invTransform(editcurve.yAxis(), pos.y())

        if self.changeConjugate:
            xData[i + 2] = xData[i + 1]
            yData[i + 2] = -yData[i + 1]
            self.__addedcZero = i + 2

        editcurve.setData(xData, yData)

        self.__addedZero = i + 1
        symbol = Qwt.QwtSymbol(editcurve.symbol())
        newSymbol = Qwt.QwtSymbol(symbol)
        newSymbol.setPen(Qt.QPen(Qt.Qt.red))
        doReplot = self.__plot.autoReplot()

        self.__plot.setAutoReplot(False)
        editcurve.setSymbol(newSymbol)

        editcurve.draw(self.__addedZero, self.__addedZero)
        if self.changeConjugate:
            editcurve.draw(self.__addedcZero, self.__addedcZero)

        editcurve.setSymbol(symbol)
        self.__plot.setAutoReplot(doReplot)
        px = []
        py = []
        for c in self.__plot.itemList():
            if isinstance(c, Qwt.QwtPlotCurve):
                px.append([c.x(i) for i in range(c.dataSize())])
                py.append([c.y(i) for i in range(c.dataSize())])
        tp = (vectorize(complex)(px[0], py[0]),
              vectorize(complex)(px[1], py[1]))
        self.curveChanged.emit(tp)
        self.__showCursor(True)

    def __showCursor(self, showIt):
        curve = self.__selectedCurve
        if not curve:
            return

        # Use copy constructors to defeat the reference semantics.
        symbol = Qwt.QwtSymbol(curve.symbol())
        newSymbol = Qwt.QwtSymbol(symbol)
        if showIt:
            newSymbol.setBrush(symbol.brush().color().dark(150))

        doReplot = self.__plot.autoReplot()

        self.__plot.setAutoReplot(False)
        curve.setSymbol(newSymbol)

        curve.draw(self.__selectedPoint, self.__selectedPoint)
        if (self.__selectedcPoint != -1):
            curve.draw(self.__selectedcPoint, self.__selectedcPoint)

        curve.setSymbol(symbol)
        self.__plot.setAutoReplot(doReplot)

    def __shiftCurveCursor(self, up):
        curves = [curve for curve in self.__plot.itemList()
                  if isinstance(curve, Qwt.QwtPlotCurve)]

        if not curves:
            return

        if self.__selectedCurve in curves:
            index = curves.index(self.__selectedCurve)
            if up:
                index += 1
            else:
                index -= 1
            # Keep index within [0, len(curves)).
            index += len(curves)
            index %= len(curves)
        else:
            index = 0

        self.__showCursor(False)
        self.__selectedPoint = 0
        self.__selectedCurve = curves[index]
        self.__showCursor(True)

    def __shiftPointCursor(self, up):
        curve = self.__selectedCurve
        if not curve:
            return

        if up:
            index = self.__selectedPoint + 1
        else:
            index = self.__selectedPoint - 1
        # Keep index within [0, curve.dataSize()).
        index += curve.dataSize()
        index %= curve.dataSize()
        if index != self.__selectedPoint:
            self.__showCursor(False)
            self.__selectedPoint = index
            self.__showCursor(True)



#
# Copyright 2024 MindSlayer001.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""Returns information about a module"""

import subprocess
import click


@click.command(name="help")
def cli():
    """
    Display help for gr_modtool (equivalent to `gr_modtool --help`).
    """
    ctx = click.get_current_context()
    click.echo(ctx.parent.get_help())



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import math


class test_vector_map(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_reversing(self):
        # Chunk data in blocks of N and reverse the block contents.
        N = 5
        src_data = list(range(0, 20))
        expected_result = []
        for i in range(N - 1, len(src_data), N):
            for j in range(0, N):
                expected_result.append(1.0 * (i - j))
        mapping = [list(reversed([(0, i) for i in range(0, N)]))]
        src = blocks.vector_source_f(src_data, False, N)
        vmap = blocks.vector_map(gr.sizeof_float, (N, ), mapping)
        dst = blocks.vector_sink_f(N)
        self.tb.connect(src, vmap, dst)
        self.tb.run()
        result_data = list(dst.data())
        self.assertEqual(expected_result, result_data)

    def test_vector_to_streams(self):
        # Split an input vector into N streams.
        N = 5
        M = 20
        src_data = list(range(0, M))
        expected_results = []
        for n in range(0, N):
            expected_results.append(list(range(n, M, N)))
        mapping = [[(0, n)] for n in range(0, N)]
        src = blocks.vector_source_f(src_data, False, N)
        vmap = blocks.vector_map(gr.sizeof_float, (N, ), mapping)
        dsts = [blocks.vector_sink_f(1) for n in range(0, N)]
        self.tb.connect(src, vmap)
        for n in range(0, N):
            self.tb.connect((vmap, n), dsts[n])
        self.tb.run()
        for n in range(0, N):
            result_data = list(dsts[n].data())
            self.assertEqual(expected_results[n], result_data)

    def test_interleaving(self):
        # Takes 3 streams (a, b and c)
        # Outputs 2 streams.
        # First (d) is interleaving of a and b.
        # Second (e) is interleaving of a and b and c.  c is taken in
        #     chunks of 2 which are reversed.
        A = [1, 2, 3, 4, 5]
        B = [11, 12, 13, 14, 15]
        C = [99, 98, 97, 96, 95, 94, 93, 92, 91, 90]
        expected_D = [1, 11, 2, 12, 3, 13, 4, 14, 5, 15]
        expected_E = [1, 11, 98, 99, 2, 12, 96, 97, 3, 13, 94, 95,
                      4, 14, 92, 93, 5, 15, 90, 91]
        mapping = [[(0, 0), (1, 0)],  # mapping to produce D
                   [(0, 0), (1, 0), (2, 1), (2, 0)],  # mapping to produce E
                   ]
        srcA = blocks.vector_source_f(A, False, 1)
        srcB = blocks.vector_source_f(B, False, 1)
        srcC = blocks.vector_source_f(C, False, 2)
        vmap = blocks.vector_map(gr.sizeof_int, (1, 1, 2), mapping)
        dstD = blocks.vector_sink_f(2)
        dstE = blocks.vector_sink_f(4)
        self.tb.connect(srcA, (vmap, 0))
        self.tb.connect(srcB, (vmap, 1))
        self.tb.connect(srcC, (vmap, 2))
        self.tb.connect((vmap, 0), dstD)
        self.tb.connect((vmap, 1), dstE)
        self.tb.run()
        self.assertEqual(expected_D, dstD.data())
        self.assertEqual(expected_E, dstE.data())


if __name__ == '__main__':
    gr_unittest.run(test_vector_map)



#!/usr/bin/env python
#
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gnuradio import gr, gr_unittest, digital, blocks
from gnuradio.gr import packet_utils
import pmt


class qa_packet_headergenerator_bb (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "tsb_key"

    def tearDown(self):
        self.tb = None

    def setup_data_tags(self, data):
        return packet_utils.packets_to_vectors(
            data,
            self.tsb_key
        )

    def test_001_12bits(self):
        # 3 packets:                       |             |       |
        data, tags = self.setup_data_tags(
            ((1, 2, 3, 4), (1, 2), tuple(range(25))))
        src = blocks.vector_source_b(data, tags=tags)
        header = digital.packet_headergenerator_bb(12, self.tsb_key)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, header, sink)
        self.tb.run()
        expected_data = [
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0
        ]
        self.assertEqual(sink.data(), expected_data)

    def test_002_32bits(self):
        # 3 packets:                       |             |       |           |
        data, tags = self.setup_data_tags(((1, 2, 3, 4), (1, 2), (1, 2, 3, 4)))
        src = blocks.vector_source_b(data, tags=tags)
        header = digital.packet_headergenerator_bb(32, self.tsb_key)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, header, sink)
        self.tb.run()
        expected_data = [
            #   | Number of symbols                  | Packet number                      | CRC
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1,
            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1,
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1
        ]
        self.assertEqual(sink.data(), expected_data)

    def test_003_12bits_formatter_object(self):
        # 3 packets:                       |             |       |           |
        data, tags = self.setup_data_tags(((1, 2, 3, 4), (1, 2), (1, 2, 3, 4)))
        src = blocks.vector_source_b(data, tags=tags)
        formatter_object = digital.packet_header_default(12, self.tsb_key)
        header = digital.packet_headergenerator_bb(
            formatter_object.formatter(), self.tsb_key)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, header, sink)
        self.tb.run()
        expected_data = [
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0
        ]
        self.assertEqual(sink.data(), expected_data)

    def test_004_8bits_formatter_ofdm(self):
        occupied_carriers = ((1, 2, 3, 5, 6, 7),)
        # 3 packets:                       |             |       |           |
        data, tags = self.setup_data_tags(((1, 2, 3, 4), (1, 2), (1, 2, 3, 4)))
        src = blocks.vector_source_b(data, tags=tags)
        formatter_object = digital.packet_header_ofdm(
            occupied_carriers, 1, self.tsb_key)
        self.assertEqual(formatter_object.header_len(), 6)
        self.assertEqual(
            pmt.symbol_to_string(
                formatter_object.len_tag_key()),
            self.tsb_key)
        header = digital.packet_headergenerator_bb(
            formatter_object.formatter(), self.tsb_key)
        sink = blocks.vector_sink_b()
        self.tb.connect(src, header, sink)
        self.tb.run()
        expected_data = [
            0, 0, 1, 0, 0, 0,
            0, 1, 0, 0, 0, 0,
            0, 0, 1, 0, 0, 0
        ]
        self.assertEqual(sink.data(), expected_data)


if __name__ == '__main__':
    gr_unittest.run(qa_packet_headergenerator_bb)




from grc.core.blocks._flags import Flags


def test_simple_init():
    assert 'test' not in Flags()
    assert 'test' in Flags(' test')
    assert 'test' in Flags('test, foo')
    assert 'test' in Flags({'test', 'foo'})


def test_deprecated():
    assert Flags.DEPRECATED == 'deprecated'
    assert Flags('this is deprecated').deprecated is True


def test_extend():
    f = Flags('a')
    f.set('b')
    assert isinstance(f, Flags)
    f.set(u'b')
    assert isinstance(f, Flags)
    f = Flags(u'a')
    f.set('b')
    assert isinstance(f, Flags)
    f.set(u'b')
    assert isinstance(f, Flags)

    assert f.data == {'a', 'b'}



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks
import math


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


def sig_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_pfb_arb_resampler(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_fff_000(self):
        N = 500         # number of samples to use
        fs = 5000.0     # baseband sampling rate
        rrate = 2.3421  # resampling rate

        nfilts = 32
        taps = filter.firdes.low_pass_2(
            nfilts,
            nfilts * fs,
            fs / 2,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 121.213
        data = sig_source_f(fs, freq, 1, N)
        signal = blocks.vector_source_f(data)
        pfb = filter.pfb_arb_resampler_fff(rrate, taps, nfilts)
        snk = blocks.vector_sink_f()

        self.tb.connect(signal, pfb, snk)
        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Get group delay and estimate of phase offset from the filter itself.
        delay = pfb.group_delay()
        phase = pfb.phase_offset(freq, fs)

        # Create a timeline offset by the filter's group delay
        t = [float(x) / (fs * rrate) for x in range(-delay, L - delay)]

        # Data of the sinusoid at frequency freq with the delay and phase
        # offset.
        expected_data = [math.sin(2. * math.pi * freq * x + phase) for x in t]

        dst_data = snk.data()

        self.assertFloatTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 2)

    def test_ccf_000(self):
        N = 5000         # number of samples to use
        fs = 5000.0      # baseband sampling rate
        rrate = 2.4321   # resampling rate

        nfilts = 32
        taps = filter.firdes.low_pass_2(
            nfilts,
            nfilts * fs,
            fs / 2,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 211.123
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        pfb = filter.pfb_arb_resampler_ccf(rrate, taps, nfilts)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, pfb, snk)
        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Get group delay and estimate of phase offset from the filter itself.
        delay = pfb.group_delay()
        phase = pfb.phase_offset(freq, fs)

        # Create a timeline offset by the filter's group delay
        t = [float(x) / (fs * rrate) for x in range(-delay, L - delay)]

        # Data of the sinusoid at frequency freq with the delay and phase
        # offset.
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 2)

    def test_ccf_001(self):
        N = 50000        # number of samples to use
        fs = 5000.0      # baseband sampling rate
        rrate = 0.75     # resampling rate

        nfilts = 32
        taps = filter.firdes.low_pass_2(
            nfilts,
            nfilts * fs,
            fs / 4,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 211.123
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        pfb = filter.pfb_arb_resampler_ccf(rrate, taps, nfilts)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, pfb, snk)
        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Get group delay and estimate of phase offset from the filter itself.
        delay = pfb.group_delay()
        phase = pfb.phase_offset(freq, fs)

        # Create a timeline offset by the filter's group delay
        t = [float(x) / (fs * rrate) for x in range(-delay, L - delay)]

        # Data of the sinusoid at frequency freq with the delay and phase
        # offset.
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 2)

    def test_ccc_000(self):
        N = 5000         # number of samples to use
        fs = 5000.0      # baseband sampling rate
        rrate = 3.4321   # resampling rate

        nfilts = 32
        taps = filter.firdes.complex_band_pass_2(
            nfilts,
            nfilts * fs,
            50,
            400,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 211.123
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        pfb = filter.pfb_arb_resampler_ccc(rrate, taps, nfilts)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, pfb, snk)
        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Get group delay and estimate of phase offset from the filter itself.
        delay = pfb.group_delay()
        phase = pfb.phase_offset(freq, fs)

        # Create a timeline offset by the filter's group delay
        t = [float(x) / (fs * rrate) for x in range(-delay, L - delay)]

        # Data of the sinusoid at frequency freq with the delay and phase
        # offset.
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 2)

    def test_ccc_001(self):
        N = 50000        # number of samples to use
        fs = 5000.0      # baseband sampling rate
        rrate = 0.715    # resampling rate

        nfilts = 32
        taps = filter.firdes.complex_band_pass_2(
            nfilts,
            nfilts * fs,
            50,
            400,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 211.123
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        pfb = filter.pfb_arb_resampler_ccc(rrate, taps, nfilts)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, pfb, snk)
        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Get group delay and estimate of phase offset from the filter itself.
        delay = pfb.group_delay()
        phase = pfb.phase_offset(freq, fs)

        # Create a timeline offset by the filter's group delay
        t = [float(x) / (fs * rrate) for x in range(-delay, L - delay)]

        # Data of the sinusoid at frequency freq with the delay and phase
        # offset.
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 2)


if __name__ == '__main__':
    gr_unittest.run(test_pfb_arb_resampler)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import time
import numpy
import os
import pmt
from gnuradio import gr, gr_unittest
from gnuradio import blocks

BLOCK_LOOKUP = {
    'float': {
        'src': blocks.vector_source_f,
        'sink': blocks.vector_sink_f,
        'mult': blocks.multiply_matrix_ff,
    },
    'complex': {
        'src': blocks.vector_source_c,
        'sink': blocks.vector_sink_c,
        'mult': blocks.multiply_matrix_cc,
    },
}


class test_multiply_matrix_xx (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.multiplier = None

    def tearDown(self):
        self.tb = None
        self.multiplier = None

    def run_once(self,
                 X_in,
                 A,
                 tpp=gr.TPP_DONT,
                 A2=None,
                 tags=None,
                 msg_A=None,
                 datatype='float',
                 ):
        """ Run the test for given input-, output- and matrix values.
        Every row from X_in is considered an input signal on a port. """
        X_in = numpy.array(X_in)
        A_matrix = numpy.array(A)
        (N, M) = A_matrix.shape
        self.assertEqual(N, X_in.shape[0])
        # Calc expected
        Y_out_exp = numpy.zeros((M, X_in.shape[1]))
        self.multiplier = BLOCK_LOOKUP[datatype]['mult'](A, tpp)
        if A2 is not None:
            self.multiplier.set_A(A2)
            A = A2
            A_matrix = numpy.array(A)
        for i in range(N):
            if tags is None:
                these_tags = ()
            else:
                these_tags = (tags[i],)
            self.tb.connect(
                BLOCK_LOOKUP[datatype]['src'](
                    X_in[i].tolist(),
                    tags=these_tags),
                (self.multiplier,
                 i))
        sinks = []
        for i in range(M):
            sinks.append(BLOCK_LOOKUP[datatype]['sink']())
            self.tb.connect((self.multiplier, i), sinks[i])
        # Run and check
        self.tb.run()
        for i in range(X_in.shape[1]):
            Y_out_exp[:, i] = numpy.matmul(A_matrix, X_in[:, i])
        Y_out = [list(x.data()) for x in sinks]
        if tags is not None:
            self.the_tags = []
            for i in range(M):
                self.the_tags.append(sinks[i].tags())
        self.assertEqual(list(Y_out), Y_out_exp.tolist())

    def test_001_t(self):
        """ Simplest possible check: N==M, unit matrix """
        X_in = (
            (1, 2, 3, 4),
            (5, 6, 7, 8),
        )
        A = (
            (1, 0),
            (0, 1),
        )
        self.run_once(X_in, A)

    def test_001_t_complex(self):
        """ Simplest possible check: N==M, unit matrix """
        X_in = (
            (1, 2, 3, 4),
            (5, 6, 7, 8),
        )
        A = (
            (1, 0),
            (0, 1),
        )
        self.run_once(X_in, A, datatype='complex')

    def test_002_t(self):
        """ Switch check: N==M, flipped unit matrix """
        X_in = (
            (1, 2, 3, 4),
            (5, 6, 7, 8),
        )
        A = (
            (0, 1),
            (1, 0),
        )
        self.run_once(X_in, A)

    def test_003_t(self):
        """ Average """
        X_in = (
            (1, 1, 1, 1),
            (2, 2, 2, 2),
        )
        A = (
            (0.5, 0.5),
            (0.5, 0.5),
        )
        self.run_once(X_in, A)

    def test_004_t(self):
        """ Set """
        X_in = (
            (1, 2, 3, 4),
            (5, 6, 7, 8),
        )
        A1 = (
            (1, 0),
            (0, 1),
        )
        A2 = (
            (0, 1),
            (1, 0),
        )
        self.run_once(X_in, A1, A2=A2)

    def test_005_t(self):
        """ Tags """
        X_in = (
            (1, 2, 3, 4),
            (5, 6, 7, 8),
        )
        A = (
            (0, 1),  # Flip them round
            (1, 0),
        )
        tag1 = gr.tag_t()
        tag1.offset = 0
        tag1.key = pmt.intern("in1")
        tag1.value = pmt.PMT_T
        tag2 = gr.tag_t()
        tag2.offset = 0
        tag2.key = pmt.intern("in2")
        tag2.value = pmt.PMT_T
        self.run_once(X_in, A, tpp=gr.TPP_ONE_TO_ONE, tags=(tag1, tag2))
        self.assertTrue(pmt.equal(tag1.key, self.the_tags[0][0].key))
        self.assertTrue(pmt.equal(tag2.key, self.the_tags[1][0].key))

    # def test_006_t (self):
        #""" Message passing """
        # X_in = (
        #(1, 2, 3, 4),
        #(5, 6, 7, 8),
        # )
        # A1 = (
        #(1, 0),
        #(0, 1),
        # )
        # msg_A = (
        #(0, 1),
        #(1, 0),
        # )
        #self.run_once(X_in, A1, msg_A=msg_A)


if __name__ == '__main__':
    gr_unittest.run(test_multiply_matrix_xx)



import os
from os.path import expanduser, normpath, expandvars, exists
from collections import OrderedDict

from ..core.Config import Config as CoreConfig
from ..main import get_config_file_path
from qtpy import QtCore


class Config(CoreConfig):

    name = 'GNU Radio Companion'

    gui_prefs_file = os.environ.get('GRC_QT_PREFS_PATH', get_config_file_path('grc_qt.conf'))

    def __init__(self, install_prefix, *args, **kwargs):
        CoreConfig.__init__(self, *args, **kwargs)
        self.install_prefix = install_prefix
        self.qsettings = QtCore.QSettings(self.gui_prefs_file, QtCore.QSettings.IniFormat)

    @property
    def wiki_block_docs_url_prefix(self):
        return self._gr_prefs.get_string('grc-docs', 'wiki_block_docs_url_prefix', '')

    @property
    def block_paths(self):
        paths_sources = (
            self.hier_block_lib_dir,
            os.environ.get('GRC_BLOCKS_PATH', ''),
            self._gr_prefs.get_string('grc', 'local_blocks_path', ''),
            self._gr_prefs.get_string('grc', 'global_blocks_path', ''),
            self.qsettings.value('grc/custom_block_paths', ''),
        )

        collected_paths = sum((paths.split(os.pathsep)
                               for paths in paths_sources), [])

        valid_paths = [normpath(expanduser(expandvars(path)))
                       for path in collected_paths if exists(path)]
        # Deduplicate paths to avoid warnings about finding blocks twice, but
        # preserve order of paths
        valid_paths = list(OrderedDict.fromkeys(valid_paths))

        return valid_paths



#!/usr/bin/env python
# Copyright (C) 2015,2016 MongoDB Inc.
# Copyright (C) 2018 Free Software Foundation
#
# This program is free software: you can redistribute it and/or  modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
A script that provides:
1. Validates clang-format is the right version.
2. Has support for checking which files are to be checked.
3. Supports validating and updating a set of files to the right coding style.
"""

import queue
import difflib
import glob
import itertools
import os
import re
import subprocess
from subprocess import check_output, CalledProcessError
import sys
import threading
import time
import shutil
from argparse import ArgumentParser
from multiprocessing import cpu_count

# Get relative imports to work when
# the package is not installed on the PYTHONPATH.
if __name__ == "__main__" and __package__ is None:
    sys.path.append(
        os.path.dirname(
            os.path.dirname(os.path.abspath(os.path.realpath(__file__)))))

##############################################################################
#
# Constants for clang-format
#
#

# Expected version of clang-format
CLANG_FORMAT_VERSION = "14.0.0"
CLANG_FORMAT_SHORT_VERSION = "14.0"

# Name of clang-format as a binary
CLANG_FORMAT_PROGNAME = "clang-format"
# only valid c/c++ implementations and headers
files_match = re.compile('\\.(h|cc|c)$')

##############################################################################


def callo(args):
    """Call a program, and capture its output
    """
    return check_output(args).decode('utf-8')


class ClangFormat(object):
    """Class encapsulates finding a suitable copy of clang-format,
    and linting/formatting an individual file
    """

    def __init__(self, path):
        self.path = None
        clang_format_progname_ext = ""

        if sys.platform == "win32":
            clang_format_progname_ext += ".exe"

        # Check the clang-format the user specified
        if path is not None:
            if os.path.isfile(path):
                self.path = path
            else:
                print("WARNING: Could not find clang-format %s" % (path))

        # Check the users' PATH environment variable now
        if self.path is None:
            # Check for various versions staring with binaries with version specific suffixes in the
            # user's path
            programs = [
                CLANG_FORMAT_PROGNAME + "-" + CLANG_FORMAT_VERSION,
                CLANG_FORMAT_PROGNAME + "-" + CLANG_FORMAT_SHORT_VERSION,
                CLANG_FORMAT_PROGNAME,
            ]

            if sys.platform == "win32":
                for i in range(len(programs)):
                    programs[i] += '.exe'

            for program in programs:
                self.path = shutil.which(program)

                if self.path:
                    if not self._validate_version():
                        self.path = None
                    else:
                        break

        # If Windows, try to grab it from Program Files
        # Check both native Program Files and WOW64 version
        if sys.platform == "win32":
            programfiles = [
                os.environ["ProgramFiles"],
                os.environ["ProgramFiles(x86)"],
            ]

            for programfile in programfiles:
                win32bin = os.path.join(programfile,
                                        "LLVM\\bin\\clang-format.exe")
                if os.path.exists(win32bin):
                    self.path = win32bin
                    break

        if self.path is None or not os.path.isfile(
                self.path) or not self._validate_version():
            print(
                "ERROR:clang-format not found in $PATH, please install clang-format " +
                CLANG_FORMAT_VERSION)
            raise NameError("No suitable clang-format found")
        self.print_lock = threading.Lock()

    def _validate_version(self):
        """Validate clang-format is the expected version
        """
        cf_version = callo([self.path, "--version"])

        if CLANG_FORMAT_VERSION in cf_version:
            return True

        print(
            "WARNING: clang-format found in path, but incorrect version found at " +
            self.path + " with version: " + cf_version)

        return False

    def _lint(self, file_name, print_diff):
        """Check the specified file has the correct format
        """
        with open(file_name, 'rb') as original_text:
            original_file = original_text.read().decode("utf-8")

        # Get formatted file as clang-format would format the file
        formatted_file = callo([self.path, "--style=file", file_name])

        if original_file != formatted_file:
            if print_diff:
                original_lines = original_file.splitlines()
                formatted_lines = formatted_file.splitlines()
                result = difflib.unified_diff(original_lines, formatted_lines)

                # Take a lock to ensure diffs do not get mixed when printed to the screen
                with self.print_lock:
                    print("ERROR: Found diff for " + file_name)
                    print("To fix formatting errors, run %s --style=file -i %s"
                          % (self.path, file_name))
                    for line in result:
                        print(line.rstrip())

            return False

        return True

    def lint(self, file_name):
        """Check the specified file has the correct format
        """
        return self._lint(file_name, print_diff=True)

    def format(self, file_name):
        """Update the format of the specified file
        """
        if self._lint(file_name, print_diff=False):
            return True

        # Update the file with clang-format
        formatted = not subprocess.call(
            [self.path, "--style=file", "-i", file_name])

        # Version 3.8 generates files like foo.cpp~RF83372177.TMP when it formats foo.cpp
        # on Windows, we must clean these up
        if sys.platform == "win32":
            glob_pattern = file_name + "*.TMP"
            for fglob in glob.glob(glob_pattern):
                os.unlink(fglob)

        return formatted


def parallel_process(items, func):
    """Run a set of work items to completion
    """
    try:
        cpus = cpu_count()
    except NotImplementedError:
        cpus = 1

    task_queue = queue.Queue()

    # Use a list so that worker function will capture this variable
    pp_event = threading.Event()
    pp_result = [True]

    def worker():
        """Worker thread to process work items in parallel
        """
        while not pp_event.is_set():
            try:
                item = task_queue.get_nowait()
            except queue.Empty:
                # if the queue is empty, exit the worker thread
                pp_event.set()
                return

            try:
                ret = func(item)
            finally:
                # Tell the queue we finished with the item
                task_queue.task_done()

            # Return early if we fail, and signal we are done
            if not ret:
                # with pp_lock:
                #     pp_result[0] = False
                print("{} failed on item {}".format(func, item))

                # pp_event.set()
                return

    # Enqueue all the work we want to process
    for item in items:
        task_queue.put(item)

    # Process all the work
    threads = []
    for cpu in range(cpus):
        thread = threading.Thread(target=worker)

        thread.daemon = True
        thread.start()
        threads.append(thread)

    # Wait for the threads to finish
    # Loop with a timeout so that we can process Ctrl-C interrupts
    # Note: On Python 2.6 wait always returns None so we check is_set also,
    #  This works because we only set the event once, and never reset it
    while not pp_event.wait(1) and not pp_event.is_set():
        time.sleep(1)

    for thread in threads:
        thread.join()

    return pp_result[0]


def get_base_dir():
    """Get the base directory for mongo repo.
        This script assumes that it is running in buildscripts/, and uses
        that to find the base directory.
    """
    try:
        return subprocess.check_output(
            ['git', 'rev-parse', '--show-toplevel']).rstrip().decode('utf-8')
    except CalledProcessError:
        # We are not in a valid git directory. Use the script path instead.
        return os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


def get_repos():
    """Get a list of Repos to check clang-format for
    """
    base_dir = get_base_dir()

    # Get a list of modules
    # GNU Radio is a single-git repo

    # paths = [os.path.join(base_dir, MODULE_DIR, m) for m in gnuradio_modules]

    paths = [base_dir]

    return [Repo(p) for p in paths]


class Repo(object):
    """Class encapsulates all knowledge about a git repository, and its metadata
        to run clang-format.
    """

    def __init__(self, path):
        self.path = path

        self.root = self._get_root()

    def _callgito(self, args):
        """Call git for this repository, and return the captured output
        """
        # These two flags are the equivalent of -C in newer versions of Git
        # but we use these to support versions pre 1.8.5 but it depends on the command
        # and what the current directory is
        return callo([
            'git', '--git-dir', os.path.join(self.path, ".git"), '--work-tree',
            self.path
        ] + args)

    def _callgit(self, args):
        """Call git for this repository without capturing output
        This is designed to be used when git returns non-zero exit codes.
        """
        # These two flags are the equivalent of -C in newer versions of Git
        # but we use these to support versions pre 1.8.5 but it depends on the command
        # and what the current directory is
        return subprocess.call([
            'git', '--git-dir', os.path.join(self.path, ".git"), '--work-tree',
            self.path
        ] + args)

    def _get_local_dir(self, path):
        """Get a directory path relative to the git root directory
        """
        if os.path.isabs(path):
            return os.path.relpath(path, self.root)
        return path

    def get_candidates(self, candidates):
        """Get the set of candidate files to check by querying the repository

        Returns the full path to the file for clang-format to consume.
        """
        if candidates is not None and len(candidates) > 0:
            candidates = [self._get_local_dir(f) for f in candidates]
            valid_files = list(
                set(candidates).intersection(self.get_candidate_files()))
        else:
            valid_files = list(self.get_candidate_files())

        # Get the full file name here
        valid_files = [
            os.path.normpath(os.path.join(self.root, f)) for f in valid_files
        ]

        return valid_files

    def get_root(self):
        """Get the root directory for this repository
        """
        return self.root

    def _get_root(self):
        """Gets the root directory for this repository from git
        """
        gito = self._callgito(['rev-parse', '--show-toplevel'])

        return gito.rstrip()

    def _git_ls_files(self, cmd):
        """Run git-ls-files and filter the list of files to a valid candidate list
        """
        gito = self._callgito(cmd)

        # This allows us to pick all the interesting files
        # in the mongo and mongo-enterprise repos
        file_list = [
            line.rstrip()
            for line in gito.splitlines()
            # TODO: exclude directories if needed
            # We don't want to lint volk
            if not "volk" in line
        ]

        file_list = [a for a in file_list if files_match.search(a)]

        return file_list

    def get_candidate_files(self):
        """Query git to get a list of all files in the repo to consider for analysis
        """
        return self._git_ls_files(["ls-files", "--cached"])

    def get_working_tree_candidate_files(self):
        """Query git to get a list of all files in the working tree to consider for analysis
        """
        return self._git_ls_files(["ls-files", "--cached", "--others"])

    def get_working_tree_candidates(self):
        """Get the set of candidate files to check by querying the repository

        Returns the full path to the file for clang-format to consume.
        """
        valid_files = list(self.get_working_tree_candidate_files())

        # Get the full file name here
        valid_files = [
            os.path.normpath(os.path.join(self.root, f)) for f in valid_files
        ]

        return valid_files

    def is_detached(self):
        """Is the current working tree in a detached HEAD state?
        """
        # symbolic-ref returns 1 if the repo is in a detached HEAD state
        return self._callgit(["symbolic-ref", "--quiet", "HEAD"])

    def is_ancestor(self, parent, child):
        """Is the specified parent hash an ancestor of child hash?
        """
        # merge base returns 0 if parent is an ancestor of child
        return not self._callgit(
            ["merge-base", "--is-ancestor", parent, child])

    def is_commit(self, sha1):
        """Is the specified hash a valid git commit?
        """
        # cat-file -e returns 0 if it is a valid hash
        return not self._callgit(["cat-file", "-e", "%s^{commit}" % sha1])

    def is_working_tree_dirty(self):
        """Does the current working tree have changes?
        """
        # diff returns 1 if the working tree has local changes
        return self._callgit(["diff", "--quiet"])

    def does_branch_exist(self, branch):
        """Does the branch exist?
        """
        # rev-parse returns 0 if the branch exists
        return not self._callgit(["rev-parse", "--verify", branch])

    def get_merge_base(self, commit):
        """Get the merge base between 'commit' and HEAD
        """
        return self._callgito(["merge-base", "HEAD", commit]).rstrip()

    def get_branch_name(self):
        """Get the current branch name, short form
           This returns "master", not "refs/head/master"
           Will not work if the current branch is detached
        """
        branch = self.rev_parse(["--abbrev-ref", "HEAD"])
        if branch == "HEAD":
            raise ValueError("Branch is currently detached")

        return branch

    def add(self, command):
        """git add wrapper
        """
        return self._callgito(["add"] + command)

    def checkout(self, command):
        """git checkout wrapper
        """
        return self._callgito(["checkout"] + command)

    def commit(self, command):
        """git commit wrapper
        """
        return self._callgito(["commit"] + command)

    def diff(self, command):
        """git diff wrapper
        """
        return self._callgito(["diff"] + command)

    def log(self, command):
        """git log wrapper
        """
        return self._callgito(["log"] + command)

    def rev_parse(self, command):
        """git rev-parse wrapper
        """
        return self._callgito(["rev-parse"] + command).rstrip()

    def rm(self, command):
        """git rm wrapper
        """
        return self._callgito(["rm"] + command)

    def show(self, command):
        """git show wrapper
        """
        return self._callgito(["show"] + command)


def get_list_from_lines(lines):
    """"Convert a string containing a series of lines into a list of strings
    """
    return [line.rstrip() for line in lines.splitlines()]


def get_files_to_check_working_tree():
    """Get a list of files to check form the working tree.
       This will pick up files not managed by git.
    """
    repos = get_repos()

    valid_files = list(
        itertools.chain.from_iterable(
            [r.get_working_tree_candidates() for r in repos]))
    return valid_files


def get_files_to_check():
    """Get a list of files that need to be checked
       based on which files are managed by git.
    """
    repos = get_repos()

    valid_files = list(
        itertools.chain.from_iterable([r.get_candidates(None) for r in repos]))

    return valid_files


def get_files_to_check_from_patch(patches):
    """
    Take a patch file generated by git diff,
    and scan the patch for a list of files to check.
    """
    candidates = []

    # Get a list of candidate_files
    check = re.compile(
        r"^diff --git a\/([a-z\/\.\-_0-9]+) b\/[a-z\/\.\-_0-9]+")

    candidates = []
    for patch in patches:
        if patch == "-":
            infile = sys.stdin
        else:
            infile = open(patch, "rb")
        candidates.extend([
            check.match(line).group(1) for line in infile.readlines()
            if check.match(line)
        ])
        infile.close()

    repos = get_repos()

    valid_files = list(
        itertools.chain.from_iterable(
            [r.get_candidates(candidates) for r in repos]))

    return valid_files


def _lint_files(clang_format, files):
    """Lint a list of files with clang-format
    """
    try:
        clang_format = ClangFormat(clang_format)
    except NameError as e:
        print(e)
        return False

    lint_clean = parallel_process([os.path.abspath(f) for f in files],
                                  clang_format.lint)

    if not lint_clean:
        print("ERROR: Code Style does not match coding style")
        sys.exit(1)


def lint(args):
    """Lint files command entry point
    """
    if args.patch and args.all:
        print("Only specify patch or all, but not both!")
        return False
    if args.patch:
        files = get_files_to_check_from_patch(args.patch)
    elif args.all:
        files = get_files_to_check_working_tree()
    else:
        files = get_files_to_check()

    if files:
        _lint_files(args.clang_format, files)

    return True


def _format_files(clang_format, files):
    """Format a list of files with clang-format
    """
    try:
        clang_format = ClangFormat(clang_format)
    except NameError as e:
        print(e)
        return (False)

    format_clean = parallel_process([os.path.abspath(f) for f in files],
                                    clang_format.format)

    if not format_clean:
        print("ERROR: failed to format files")
        sys.exit(1)


def _reformat_branch(clang_format, commit_prior_to_reformat,
                     commit_after_reformat):
    """Reformat a branch made before a clang-format run
    """
    try:
        clang_format = ClangFormat(clang_format)
    except NameError as e:
        print(e)
        return False

    if os.getcwd() != get_base_dir():
        raise ValueError("reformat-branch must be run from the repo root")

    repo = Repo(get_base_dir())

    # Validate that user passes valid commits
    if not repo.is_commit(commit_prior_to_reformat):
        raise ValueError(
            "Commit Prior to Reformat '%s' is not a valid commit in this repo"
            % commit_prior_to_reformat)

    if not repo.is_commit(commit_after_reformat):
        raise ValueError(
            "Commit After Reformat '%s' is not a valid commit in this repo" %
            commit_after_reformat)

    if not repo.is_ancestor(commit_prior_to_reformat, commit_after_reformat):
        raise ValueError((
            "Commit Prior to Reformat '%s' is not a valid ancestor of Commit After" +
            " Reformat '%s' in this repo") % (commit_prior_to_reformat,
                                              commit_after_reformat))

    # Validate the user is on a local branch that has the right merge base
    if repo.is_detached():
        raise ValueError(
            "You must not run this script in a detached HEAD state")

    # Validate the user has no pending changes
    if repo.is_working_tree_dirty():
        raise ValueError(
            "Your working tree has pending changes. You must have a clean working tree before proceeding."
        )

    merge_base = repo.get_merge_base(commit_prior_to_reformat)

    if not merge_base == commit_prior_to_reformat:
        raise ValueError(
            "Please rebase to '%s' and resolve all conflicts before running this script"
            % (commit_prior_to_reformat))

    # We assume the target branch is master, it could be a different branch if needed for testing
    merge_base = repo.get_merge_base("master")

    if not merge_base == commit_prior_to_reformat:
        raise ValueError(
            "This branch appears to already have advanced too far through the merge process"
        )

    # Everything looks good so lets start going through all the commits
    branch_name = repo.get_branch_name()
    new_branch = "%s-reformatted" % branch_name

    if repo.does_branch_exist(new_branch):
        raise ValueError(
            "The branch '%s' already exists. Please delete the branch '%s', or rename the current branch."
            % (new_branch, new_branch))

    commits = get_list_from_lines(
        repo.log([
            "--reverse", "--pretty=format:%H", "%s..HEAD" %
            commit_prior_to_reformat
        ]))

    previous_commit_base = commit_after_reformat

    # Go through all the commits the user made on the local branch and migrate to a new branch
    # that is based on post_reformat commits instead
    for commit_hash in commits:
        repo.checkout(["--quiet", commit_hash])

        deleted_files = []

        # Format each of the files by checking out just a single commit from the user's branch
        commit_files = get_list_from_lines(repo.diff(["HEAD~", "--name-only"]))

        for commit_file in commit_files:

            # Format each file needed if it was not deleted
            if not os.path.exists(commit_file):
                print(
                    "Skipping file '%s' since it has been deleted in commit '%s'"
                    % (commit_file, commit_hash))
                deleted_files.append(commit_file)
                continue

            if files_match.search(commit_file):
                clang_format.format(commit_file)
            else:
                print(
                    "Skipping file '%s' since it is not a file clang_format should format"
                    % commit_file)

        # Check if anything needed reformatting, and if so amend the commit
        if not repo.is_working_tree_dirty():
            print("Commit %s needed no reformatting" % commit_hash)
        else:
            repo.commit(["--all", "--amend", "--no-edit"])

        # Rebase our new commit on top the post-reformat commit
        previous_commit = repo.rev_parse(["HEAD"])

        # Checkout the new branch with the reformatted commits
        # Note: we will not name as a branch until we are done with all commits on the local branch
        repo.checkout(["--quiet", previous_commit_base])

        # Copy each file from the reformatted commit on top of the post reformat
        diff_files = get_list_from_lines(
            repo.diff([
                "%s~..%s" % (previous_commit, previous_commit), "--name-only"
            ]))

        for diff_file in diff_files:
            # If the file was deleted in the commit we are reformatting, we need to delete it again
            if diff_file in deleted_files:
                repo.rm([diff_file])
                continue

            if "volk" in diff_file:
                continue
            # The file has been added or modified, continue as normal
            file_contents = repo.show(["%s:%s" % (previous_commit, diff_file)])

            root_dir = os.path.dirname(diff_file)
            if root_dir and not os.path.exists(root_dir):
                os.makedirs(root_dir)

            with open(diff_file, "w+") as new_file:
                new_file.write(file_contents)

            repo.add([diff_file])

        # Create a new commit onto clang-formatted branch
        repo.commit(["--reuse-message=%s" % previous_commit])

        previous_commit_base = repo.rev_parse(["HEAD"])

    # Create a new branch to mark the hashes we have been using
    repo.checkout(["-b", new_branch])

    print("reformat-branch is done running.\n")
    print(
        "A copy of your branch has been made named '%s', and formatted with clang-format.\n"
        % new_branch)
    print("The original branch has been left unchanged.")
    print("The next step is to rebase the new branch on 'master'.")


def format_func(args):
    """Format files command entry point
    """
    if args.all and args.branch is not None:
        print("Only specify branch or all, but not both!")
        return False
    if not args.branch:
        if args.all:
            files = get_files_to_check_working_tree()
        else:
            files = get_files_to_check()
        _format_files(args.clang_format, files)
    else:
        _reformat_branch(args.clang_format, *args.branch)


def parse_args():
    """
    Parse commandline arguments
    """
    parser = ArgumentParser()
    parser.add_argument(
        "-c",
        "--clang-format",
        default="clang-format",
        help="clang-format binary")
    subparsers = parser.add_subparsers(
        help="clang-format action", dest="action")
    subparsers.required = True
    lint_parser = subparsers.add_parser(
        "lint", help="Lint-only (no modifications)")
    lint_parser.add_argument("-a", "--all", action="store_true")
    lint_parser.add_argument("-p", "--patch", help="patch to check")
    lint_parser.set_defaults(func=lint)

    format_parser = subparsers.add_parser(
        "format", help="Format files in place")
    format_parser.add_argument(
        "-b",
        "--branch",
        nargs=2,
        default=None,
        help="specify the commit hash before the format and after the format has been done"
    )
    format_parser.add_argument("-a", "--all", action="store_true")
    format_parser.set_defaults(func=format_func)
    return parser.parse_args()


def main():
    """Main entry point
    """
    args = parse_args()
    if hasattr(args, "func"):
        args.func(args)


if __name__ == "__main__":
    main()
